@@ -27,6 +27,7 @@ open CommonClient
 open CommonServer
 open CommonNetwork
 open CommonOptions
+open CommonUserDb
 open CommonTypes
 open CommonFile
 open Gettext
@@ -102,6 +103,61 @@ module FileOption = struct
               impl.impl_file_age <- 
                 normalize_time (get_value "file_age" value_to_int)
             with _ -> ());
+
+          (try
+              impl.impl_file_release <- 
+                get_value "file_release" value_to_bool
+            with _ -> ());
+
+	  let file_user =
+	    let filename = get_value "file_filename" value_to_string in
+	    try
+	      let u = get_value "file_owner" value_to_string in
+		begin
+		  try
+		    user2_user_find u
+		  with Not_found ->
+		    lprintf_nl "file_owner %s of %s does not exist, changing to %s"
+		      u filename admin_user.user_name;
+		    admin_user
+		end
+	    with Not_found ->
+		lprintf_nl "file_owner of %s is empty, changing to %s"
+		  filename admin_user.user_name;
+		admin_user
+	  in
+	  set_file_owner file file_user;
+
+	  let file_group =
+	    let filename = get_value "file_filename" value_to_string in
+	    let dgroup = user2_print_user_default_group file_user in
+	    try
+	      match (get_value "file_group" stringvalue_to_option) with
+		None -> None
+	      | Some g ->
+		  begin
+		    try
+		      let g = user2_group_find g in
+		      if List.mem g file_user.user_groups then
+		        Some g
+		      else
+			begin
+			  lprintf_nl "file_owner %s is not member of file_group %s, changing file_group of %s to user_default_group %s"
+			    file_user.user_name g.group_name filename dgroup;
+			  file_user.user_default_group
+			end
+		    with Not_found ->
+		      lprintf_nl "file_group %s of %s not found, changing to user_default_group %s of user %s"
+			g filename dgroup file_user.user_name;
+		      file_user.user_default_group
+		  end
+	    with Not_found ->
+		lprintf_nl "file_group of %s is empty, changing to user_default_group %s of user %s"
+		  filename dgroup file_user.user_name;
+		file_user.user_default_group
+	  in
+	  set_file_group file file_group;
+
           set_file_state file file_state;       
 
           (try
@@ -138,6 +194,9 @@ module FileOption = struct
 	("file_filenames", List
 	(List.map string_to_value impl.impl_file_filenames)) ::
         ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
+        ("file_release", bool_to_value impl.impl_file_release) ::
+        ("file_owner", string_to_value (file_owner file).user_name) ::
+        ("file_group", option_to_stringvalue (match file_group file with Some g -> Some g.group_name | None -> None)) ::
           (file_to_option file)
         )
           
@@ -512,7 +571,7 @@ let special_queries = define_option searches_section
     "-movies", "avi -minsize 650000000 -1cd";
     "-mp3s", "mp3 -minsize 3000000 -maxsize 10000000";
     "-albums", "album -minsize 30000000 -maxsize 150000000";
-    "-nosex", "-without xxx";
+    "-nosex", "-not xxx";
   ]
   
 let customized_queries =
@@ -664,7 +723,7 @@ let sharing_only_directory = {
     sharing_directories = false;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -673,7 +732,7 @@ let sharing_incoming_directories = {
     sharing_directories = true;
     sharing_extensions = [];
     sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }  
   
@@ -681,8 +740,8 @@ let sharing_incoming_files = {
     sharing_incoming = true;
     sharing_directories = false;
     sharing_extensions = [];
-    sharing_recursive = false;
-    sharing_minsize = Int64.one;
+    sharing_recursive = true;
+    sharing_minsize = zero;
     sharing_maxsize = Int64.max_int;
   }
 
@@ -691,7 +750,7 @@ let sharing_directories = {
       sharing_directories = true;
       sharing_extensions = [];
       sharing_recursive = false;
-      sharing_minsize = Int64.one;
+      sharing_minsize = zero;
       sharing_maxsize = Int64.max_int;
     }
   
@@ -726,7 +785,7 @@ let sharing_strategies = define_option searches_section
       sharing_directories = false;
       sharing_extensions = [];
       sharing_recursive = true;
-      sharing_minsize = Int64.one;
+      sharing_minsize = zero;
       sharing_maxsize = Int64.max_int;
     };
     
@@ -758,7 +817,7 @@ let _ =
       
   )
 
-let sharing_strategies name =
+let sharing_strategy name =
   match name with
   | "incoming_files" -> sharing_incoming_files
   | "incoming_directories" -> sharing_incoming_directories
@@ -790,16 +849,18 @@ module SharedDirectoryOption = struct
             in
             let shdir_priority = get_value_safe "priority" value_to_int 0
             in
+(*
             let shdir_networks = get_value_safe "networks"
                 (value_to_list value_to_string) [] 
             in
+*)
             let shdir_strategy = get_value_safe "strategy"
                 value_to_string "only_directory"
             in
             {
               shdir_dirname = shdir_dirname; 
               shdir_strategy = shdir_strategy; 
-              shdir_networks = shdir_networks; 
+              shdir_networks = []; (* shdir_networks; *)
               shdir_priority = shdir_priority; 
             }
           end
@@ -825,8 +886,10 @@ module SharedDirectoryOption = struct
     let shared_directory_to_value s =
       let list = [
           "dirname", filename_to_value s.shdir_dirname;
+(*
           "networks", 
           list_to_value string_to_value s.shdir_networks;
+*)
           "strategy", 
           string_to_value  s.shdir_strategy;
           "priority", int_to_value s.shdir_priority;
@@ -839,8 +902,21 @@ module SharedDirectoryOption = struct
         shared_directory_to_value
 
   end
-    
-  
+
+let default_incoming_files = {
+  shdir_dirname = Filename.concat "incoming" "files";
+  shdir_priority = 0;
+  shdir_networks = [];
+  shdir_strategy = "incoming_files";
+  }
+
+let default_incoming_directories = {
+  shdir_dirname = Filename.concat "incoming" "directories";
+  shdir_priority = 0;
+  shdir_networks = [];
+  shdir_strategy = "incoming_directories";
+  }
+
 let shared_directories = 
   define_option CommonOptions.path_section ["shared_directories" ] 
   "                Incoming and shared directories.
@@ -852,8 +928,8 @@ let shared_directories =
    Finished BT multifile downloads are committed to the first directory
    with strategy incoming_directories. Other downloads are committed
    to the first directory with the strategy incoming_files.
-   If more than one directory has one of the incoming_* strategies
-   it will be ignored on commit, but they are shared nonetheless.
+   MLdonkey searches all shared_directories with incoming_* strategies
+   on commit and uses the first one with enough free diskspace.
    Other strategies can be found in searches.ini, section customized_sharing."
     (list_option SharedDirectoryOption.t) 
   [
@@ -863,64 +939,92 @@ let shared_directories =
       shdir_networks = [];
       shdir_strategy = "all_files";
     };
-    {
-      shdir_dirname = "incoming/files";
-      shdir_priority = 0;
-      shdir_networks = [];
-      shdir_strategy = "incoming_files";
-    };
-    {
-      shdir_dirname = "incoming/directories";
-      shdir_priority = 0;
-      shdir_networks = [];
-      shdir_strategy = "incoming_directories";
-    }
+    default_incoming_files;
+    default_incoming_directories;
   ]
 
 
 let search_incoming_files () =
-  try
-    List.find (fun s -> s.shdir_strategy = "incoming_files") 
-    !!shared_directories
-  with Not_found ->
-        let dirname = Filename.concat "incoming" "files" in
-        let s = {
-        shdir_dirname = dirname;
-        shdir_priority = 0;
-        shdir_networks = [];
-        shdir_strategy = "incoming_files";
-          }
-        in
-        shared_directories =:= s :: !!shared_directories;
-        s
-
-let incoming_files () =
-  let dir = search_incoming_files () in
-    Unix2.safe_mkdir dir.shdir_dirname;
-    Unix2.can_write_to_directory dir.shdir_dirname;
-    dir
+  let list =
+    List.filter (fun s -> s.shdir_strategy = "incoming_files") !!shared_directories
+  in
+  match list with
+  | [] -> shared_directories =:= default_incoming_files :: !!shared_directories;
+        [default_incoming_files]
+  | l -> l
 
 let search_incoming_directories () =
-  try
-    List.find (fun s -> s.shdir_strategy = "incoming_directories") 
-    !!shared_directories
-  with Not_found ->
-      let dirname = Filename.concat "incoming" "directories" in
-      let s = {
-          shdir_dirname = dirname;
-          shdir_priority = 0;
-          shdir_networks = [];
-          shdir_strategy = "incoming_directories";
-        }
-      in
-      shared_directories =:= s :: !!shared_directories;
-      s
+  let list =
+    List.filter (fun s -> s.shdir_strategy = "incoming_directories") !!shared_directories
+  in
+  match list with
+  | [] -> shared_directories =:= default_incoming_directories :: !!shared_directories;
+        [default_incoming_directories]
+  | l -> l
+
+exception Incoming_full
+
+let incoming_dir usedir ?user ?needed_space ?network () =
+
+  let directories =
+    if usedir then
+      search_incoming_directories ()
+    else
+      search_incoming_files ()
+  in
+
+  let dirname_user =
+    match user with
+    | None -> ""
+    | Some user -> user.user_commit_dir
+  in
+
+(*
+  let dirname_network =
+    match network with
+    | None -> ""
+    | Some network -> network
+  in
+*)
+(* todo: make the dir naming order user configurable *)
+  let compute_dir_name dir =
+    let dirname = Filename2.normalize (Filename.concat dir dirname_user) in
+(*    let dirname = Filename.concat dirname dirname_network in *)
+    dirname
+  in
+
+  let checkdir =
+    try
+      List.find (fun d ->
+	let dirname = compute_dir_name d.shdir_dirname in
+(* check if temp_directory and incoming are on different partitions *)
+	try
+          if (Unix.stat dirname).Unix.st_dev <> (Unix.stat !!temp_directory).Unix.st_dev then
+            begin
+              match needed_space with
+              | None -> true
+              | Some needed_space ->
+                  match Unix32.diskfree dirname with
+                    Some v -> v >= needed_space
+	          | _ -> true
+            end
+          else true
+	with _ -> true
+        ) directories
+    with Not_found -> raise Incoming_full;
+  in
+
+  let newdir = {
+      shdir_dirname = (compute_dir_name checkdir.shdir_dirname);
+      shdir_priority = checkdir.shdir_priority;
+      shdir_networks = checkdir.shdir_networks;
+      shdir_strategy = checkdir.shdir_strategy;
+    }
+  in
+  Unix2.safe_mkdir newdir.shdir_dirname;
+  Unix2.can_write_to_directory newdir.shdir_dirname;
+  newdir
 
-let incoming_directories () =
-  let dir = search_incoming_directories () in
-    Unix2.safe_mkdir dir.shdir_dirname;
-    Unix2.can_write_to_directory dir.shdir_dirname;
-    dir
 
 let _ =
 (* Check the definition of the incoming_files and incoming_directories in
@@ -929,8 +1033,8 @@ shared_directories *)
   option_hook shared_directories (fun _ ->
       if not !verification then begin
           verification := true;
-          ignore (incoming_files ());
-          ignore (incoming_directories ());
+          ignore (incoming_dir false ());
+          ignore (incoming_dir true ());
           verification := false
         end
   )
@@ -1038,19 +1142,15 @@ let backup_tar archive files =
 	    Pervasives.really_input ic s 0 size;
 	    header, s) in
 	Tar.output otar header s
-      with e ->
-	let error = Printexc2.to_string e in
-	if error = "Gzip.Error(\"error during compression\")"
-	  && Autoconf.windows && arg = "fasttrack.ini" then begin
-	    (* for whatever reason this error is raised on Windows,
-               but fasttrack.ini is stored correctly *)
-	    if !verbose then
-        lprintf_nl "Tar: Windows specific pseudo error %s in %s" error arg
-	  end
-	else begin
+      with
+      | (Gzip.Error "error during compression") as e when Autoconf.windows && arg = "fasttrack.ini" ->
+	  (* for whatever reason this error is raised on Windows,
+             but fasttrack.ini is stored correctly *)
+	  if !verbose then
+            lprintf_nl "Tar: Windows specific pseudo error %s in %s" (Printexc2.to_string e) arg
+      | e ->
 	  failed_files := arg :: !failed_files;
-    lprintf_nl "Tar: error %s in %s" error arg
-        end
+	  lprintf_nl "Tar: error %s in %s" (Printexc2.to_string e) arg
     ) files);
     if !failed_files <> [] then
       failwith (Printf.sprintf "Tar: error backing up %s"
@@ -1141,7 +1241,7 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
   let reserved_fds = 40 in (* ini files, dynamic libs, etc. *)
 
   let total_files = (* maximum number of files in use at the same time *)
-    (maxi (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
+    (max (List.length !!files) !!max_concurrent_downloads) + !!max_upload_slots + reserved_fds
   in
 
   let wanted_socks = !!max_opened_connections + total_files in
@@ -1158,7 +1258,7 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2 *)
   else
     begin
       let new_max_opened_connections =
-        maxi (max_all_sockets - total_files) (max_all_sockets / 2)
+        max (max_all_sockets - total_files) (max_all_sockets / 2)
       in
       lprintf_nl "max_opened_connections is set too high (%d), reducing to %d"
         !!max_opened_connections new_max_opened_connections;
