@@ -25,6 +25,7 @@ open Options
 open CommonTypes
 open CommonOptions
 open CommonGlobals
+open CommonUserDb
 
 let log_prefix = "[cF]"
 
@@ -41,10 +42,12 @@ let lprintf_n fmt =
 (*************************************************************************)
 
 type 'a file_impl = {
+    mutable impl_file_owner : userdb;
+    mutable impl_file_group : groupdb option;
     mutable impl_file_update : int;
     mutable impl_file_state : file_state;
 
-    mutable impl_file_comment : string list;
+    mutable impl_file_comment : string;
     mutable impl_file_num : int;
     mutable impl_file_val : 'a;
     mutable impl_file_ops : 'a file_ops;
@@ -59,6 +62,7 @@ type 'a file_impl = {
     mutable impl_file_filenames : string list;
     mutable impl_file_magic : string option;
     mutable impl_file_priority: int; (* normal = 0, low < 0, high > 0 *)
+    mutable impl_file_release : bool;
     mutable impl_file_last_seen : int;
     mutable impl_file_probable_name : string option;
   }
@@ -75,6 +79,7 @@ section, and thus, before it is shared. *)
 has been changed. The method should not perform the move, just know that
 it will happen soon. *)
     mutable op_file_save_as : ('a -> string -> unit);
+    mutable op_file_shared : ('a -> CommonTypes.shared option);
     mutable op_file_to_option : ('a -> (string * option_value) list);
     mutable op_file_cancel : ('a -> unit);
     mutable op_file_pause : ('a -> unit);
@@ -85,9 +90,10 @@ it will happen soon. *)
     mutable op_file_recover : ('a -> unit);
     mutable op_file_all_sources : ('a -> client list);
     mutable op_file_active_sources : ('a -> client list);
+    mutable op_file_comment : ('a -> string);
     mutable op_file_set_priority : ('a -> int -> unit);
-    mutable op_file_print_html : ('a -> Buffer.t -> unit);
-    mutable op_file_print_sources_html : ('a -> Buffer.t -> unit);
+    mutable op_file_print : ('a -> CommonTypes.ui_conn -> unit);
+    mutable op_file_print_sources : ('a -> CommonTypes.ui_conn -> unit);
     mutable op_file_files : ('a -> 'a file_impl -> file list);    
 (* added in 2.5.27 to remove use of network names in global modules *)
     mutable op_file_debug : ('a -> string);
@@ -130,9 +136,12 @@ let dummy_file_impl = {
     impl_file_filenames = [];
     impl_file_magic = None;
     impl_file_priority = 0;
+    impl_file_release = false;
     impl_file_last_seen = 0;
-    impl_file_comment = [];
+    impl_file_comment = "";
     impl_file_probable_name = None;
+    impl_file_owner = admin_user;
+    impl_file_group = Some system_user_default_group;
   }
 
 let dummy_file = as_file dummy_file_impl
@@ -198,16 +207,18 @@ let file_to_option (file : file) =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_to_option file.impl_file_val
 
-  (*
-let file_print (file : file) buf =
-  let file = as_file_impl file in
-  file.impl_file_ops.op_file_print file.impl_file_val buf
-    *)
-
 let file_save_as (file : file) name =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_save_as file.impl_file_val name
 
+let file_shared (file : file) =
+  let file = as_file_impl file in
+  file.impl_file_ops.op_file_shared file.impl_file_val
+
+let file_comment (file : file) =
+  let file = as_file_impl file in
+  file.impl_file_ops.op_file_comment file.impl_file_val
+
 let file_network (file : file) =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_network
@@ -216,7 +227,17 @@ let file_info (file : file) =
   let file = as_file_impl file in
   file.impl_file_ops.op_file_info file.impl_file_val
 
-let file_pause (file : file) =
+let file_owner file =
+  (as_file_impl file).impl_file_owner
+
+let file_group file =
+  (as_file_impl file).impl_file_group
+
+let user2_allow_file_admin file user =
+  user2_is_admin user || file_owner file = user
+
+let file_pause (file : file) user =
+  if user2_allow_file_admin file user then
   let file = as_file_impl file in
   match file.impl_file_state with
   | FileDownloading | FileQueued ->
@@ -224,7 +245,8 @@ let file_pause (file : file) =
       file.impl_file_ops.op_file_pause file.impl_file_val
   | _ -> ()
 
-let file_resume (file : file) =
+let file_resume (file : file) user =
+  if user2_allow_file_admin file user then
   let file = as_file_impl file in
   match file.impl_file_state with
   | FilePaused | FileAborted _ ->
@@ -232,20 +254,47 @@ let file_resume (file : file) =
       file.impl_file_ops.op_file_resume file.impl_file_val
   | _ -> ()
 
+let set_file_release file status user =
+  if user2_allow_file_admin file user then
+  let impl = as_file_impl file in
+  impl.impl_file_release <- status
+
+let file_release (file: file) =
+  let impl = as_file_impl file in
+  impl.impl_file_release
+
+let set_file_owner file owner =
+  (as_file_impl file).impl_file_owner <- owner
+
+let set_file_group file group =
+  (as_file_impl file).impl_file_group <- group
+
+let user2_filter_files files gui_user =
+  let newlist = List.filter
+    (fun file -> user2_can_view_file gui_user (file_owner file) (file_group file)) files in
+  newlist
+
+let user2_num_user_dls user =
+  let n = ref 0 in
+  H.iter (fun f -> if file_owner f = user then incr n) files_by_num;
+  !n
+
+let user2_num_group_dls group =
+  let n = ref 0 in
+  H.iter (fun f -> if file_group f = Some group then incr n) files_by_num;
+  !n
+
 let set_file_state file state =
   let impl = as_file_impl file in
   update_file_state impl state
 
 let set_file_comment file comment =
-  if not (List.mem comment (as_file_impl file).impl_file_comment) then
-  (as_file_impl file).impl_file_comment <-
-    (as_file_impl file).impl_file_comment @ [(HashComments.merge file_comments comment)]
+  let impl = as_file_impl file in
+  impl.impl_file_comment <- comment
 
 let file_comment file =
-  (as_file_impl file).impl_file_comment
-
-let file_comment_length file =
-  List.length (as_file_impl file).impl_file_comment
+  let impl = as_file_impl file in
+  impl.impl_file_comment
 
 let file_best_name (file : file) =
   let file = as_file_impl file in
@@ -316,7 +365,9 @@ let file_active_sources file =
   try impl.impl_file_ops.op_file_active_sources impl.impl_file_val with _ -> []
 
 (* Default for networks that don't implement it *)
-let default_file_print_sources_html file buf =
+let default_file_print_sources file o =
+  let buf = o.conn_buf in
+  if use_html_mods o then begin
   let cfile = as_file file in
   let allsources = ref (file_all_sources cfile) in
   if List.length !allsources > 0 then begin
@@ -346,7 +397,8 @@ let default_file_print_sources_html file buf =
           ("", "sr br ar", Printf.sprintf "%d" (client_num c));
           ("", "sr br", cinfo.GuiTypes.client_name);
           ("", "sr br", addr);
-          ("", "sr br", cinfo.GuiTypes.client_software);
+          (GuiTypes.client_software cinfo.GuiTypes.client_software cinfo.GuiTypes.client_os,
+	   "sr br", GuiTypes.client_software_short cinfo.GuiTypes.client_software cinfo.GuiTypes.client_os);
           ("", "sr ar", (size_of_int64 cinfo.GuiTypes.client_uploaded));
           ("", "sr ar br", (size_of_int64 cinfo.GuiTypes.client_downloaded)); ];
 
@@ -357,16 +409,26 @@ let default_file_print_sources_html file buf =
     Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
 
   end
+  end
+  else
+    let cfile = as_file file in
+    let srcs = ref (file_all_sources cfile) in
+      Printf.bprintf buf "%d sources:\n" (List.length !srcs);
+    let print_source c =
+      Printf.bprintf buf "  [%4d] " (client_num c);
+      client_bprint c buf;
+    in
+    List.iter print_source !srcs;
+    ()
 
-
-let file_print_sources_html (file : file) buf =
+let file_print_sources (file : file) conn =
   let file = as_file_impl file in
-  try file.impl_file_ops.op_file_print_sources_html file.impl_file_val buf with _ ->
-    default_file_print_sources_html file buf
+  try file.impl_file_ops.op_file_print_sources file.impl_file_val conn with _ ->
+    default_file_print_sources file conn
 
-let file_print_html file buf =
+let file_print file o =
   let impl = as_file_impl file in
-  impl.impl_file_ops.op_file_print_html impl.impl_file_val buf
+  impl.impl_file_ops.op_file_print impl.impl_file_val o
 
 let file_find num =
   H.find files_by_num (as_file {
@@ -469,7 +531,9 @@ let file_magic file =
 let set_file_magic file magic =
   match magic with
     None -> ()
-  | Some magic -> (as_file_impl file).impl_file_magic <- Some (HashMagic.merge files_magic magic)
+  | Some magic -> 
+    (as_file_impl file).impl_file_magic <- Some (intern magic);
+    file_must_update file
 
 let check_magic file =
   let check file =
@@ -488,9 +552,10 @@ let set_file_last_seen file age =
   impl.impl_file_last_seen <- age
     
 let file_preview (file : file) =
-  let cmd = Printf.sprintf "%s \"%s\" \"%s\"" !!previewer
-	      (file_disk_name file) (file_best_name file) in
-  ignore (Sys.command cmd)
+  ignore(
+    Unix.create_process !!previewer 
+      [| Filename2.basename !!previewer; file_disk_name file; file_best_name file |]
+      Unix.stdin Unix.stdout Unix.stderr)
 
 (*************************************************************************)
 (*                                                                       *)
@@ -626,6 +691,77 @@ let file_print file o =
         ("File priority", "sr br", "Priority");
         ("", "sr", Printf.sprintf "%d" (file_priority file)) ];
 
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      if user2_allow_file_admin file o.conn_user.ui_user then
+        let optionlist = ref "" in
+        user2_users_iter (fun user ->
+	  if user <> (file_owner file) then
+            optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\\</option\\>\n" user.user_name user.user_name;
+        );
+
+        html_mods_td buf [("Change file owner by selecting an alternate user", "sr br", "User");
+          ("Change owner", "sr", Printf.sprintf "
+\\<script type=\\\"text/javascript\\\"\\>
+\\<!--
+function submitChownForm(i) {
+var formID = document.getElementById(\\\"chownForm\\\" + i)
+var v = formID.newOwner.value;
+parent.fstatus.location.href='submit?q=chown+'+v+'+%d';
+}
+//--\\>
+\\</script\\>" (file_num file)
+   ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>"
+   ^ "\\<form name=\\\"chownForm1\\\" id=\\\"chownForm1\\\" action=\\\"javascript:submitChownForm(1);\\\"\\>"
+   ^ "\\<td\\>"
+   ^ "\\<select name=\\\"newOwner\\\" id=\\\"newOwner\\\" "
+   ^ "style=\\\"padding: 0px; font-size: 10px; font-family: verdana\\\" onchange=\\\"this.form.submit()\\\"\\>"
+   ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\\</option\\>\n" (file_owner file).user_name (file_owner file).user_name
+   ^ !optionlist ^ "\\</select\\>\\</td\\>\\</form\\>\\</tr\\>\\</table\\>"
+    ) ];
+
+      else
+        html_mods_td buf [("File owner", "sr br", "User"); ("", "sr", (file_owner file).user_name)];
+
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      if user2_allow_file_admin file o.conn_user.ui_user &&
+         o.conn_user.ui_user.user_groups <> [] then
+        let optionlist =
+          if (file_group file) = None then
+            ref ""
+          else
+            ref "\\<option value=\\\"None\\\"\\>None\\</option\\>\n"
+        in
+        user2_user_groups_iter (file_owner file) (fun group ->
+	  if Some group <> (file_group file) then
+            optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\\</option\\>\n" group.group_name group.group_name;
+        );
+
+        html_mods_td buf [("Change file group by selecting an alternate group", "sr br", "Group");
+          ("Change group", "sr", Printf.sprintf "
+\\<script type=\\\"text/javascript\\\"\\>
+\\<!--
+function submitChgrpForm(i) {
+var formID = document.getElementById(\\\"chgrpForm\\\" + i)
+var v = formID.newGroup.value;
+parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
+}
+//--\\>
+\\</script\\>" (file_num file)
+   ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>"
+   ^ "\\<form name=\\\"chgrpForm1\\\" id=\\\"chgrpForm1\\\" action=\\\"javascript:submitChgrpForm(1);\\\"\\>"
+   ^ "\\<td\\>"
+   ^ "\\<select name=\\\"newGroup\\\" id=\\\"newGroup\\\" "
+   ^ "style=\\\"padding: 0px; font-size: 10px; font-family: verdana\\\" onchange=\\\"this.form.submit()\\\"\\>"
+   ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\\</option\\>\n" (user2_print_group (file_group file)) (user2_print_group (file_group file))
+   ^ !optionlist ^ "\\</select\\>\\</td\\>\\</form\\>\\</tr\\>\\</table\\>"
+  ) ];
+
+      else
+        html_mods_td buf [("File group", "sr br", "Group");
+        ("", "sr", (match file_group file with
+          Some group -> Printf.sprintf "%s" group.group_name
+        | None -> "None"))];
+
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("Number of file sources", "sr br", "Sources");
@@ -671,30 +807,14 @@ let file_print file o =
             ("", "sr", magic) ]
        | _ -> ());
 
-      (
-	if file_comment file <> [] then
-	  begin
-	    let list_header = ref true in
-	    List.iter (fun s ->
-	      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-	      if !list_header then html_mods_td buf [
-		("File comments", "sr br", "File comments");
-		("", "sr", s) ]
-	      else
-	        html_mods_td buf [
-		("", "sr br", "");
-		("", "sr", s) ];
-	    if !list_header then list_header := false) (file_comment file);
-	  end);
-
-      file_print_html file buf;
+      file_print file o;
       
       Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"; 
       Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>\\<br\\>";
 
     end else
     begin
-      Printf.bprintf buf "[%-s %5d]\n%s\n%s%s\nTotal   %10s\nPartial %10s\npriority %d\n"
+      Printf.bprintf buf "[%-s %5d]\n%s\n%s%s\nTotal   %10s\nPartial %10s\npriority %d\nOwner/Group: %s/%s\n"
         n.network_name
         (file_num file)
         (shorten (file_best_name file) 80)
@@ -704,7 +824,11 @@ let file_print file o =
         (string_of_uids info.G.file_uids)
         (Int64.to_string info.G.file_size)
         (Int64.to_string info.G.file_downloaded)
-        (file_priority file);
+        (file_priority file)
+        (file_owner file).user_name
+	(match file_group file with
+	   Some group -> Printf.sprintf "%s" group.group_name
+	 | None -> "private");
       Printf.bprintf buf "Chunks: [%-s]\n"
 	(match info.G.file_chunks with
 	| None -> ""
@@ -713,24 +837,12 @@ let file_print file o =
           None -> ()
         | Some filename ->
             Printf.bprintf buf "Probable name: %s\n" filename);
-      List.iter (fun name -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names
+      List.iter (fun name -> Printf.bprintf buf "    (%s)\n" name) info.G.file_names;
+      file_print file o
     end;
 
   (try
-            
-      if !!print_all_sources then begin
-          if use_html_mods o then
-            file_print_sources_html file buf
-          else begin
-            Printf.bprintf buf "%d sources:\n" (List.length srcs);
-          let print_source c =
-                  Printf.bprintf buf "  [%4d] " (client_num c);
-                  client_bprint c buf;
-          in
-          List.iter print_source srcs;
-        end;
-        end
-    
+      if !!print_all_sources then file_print_sources file o
     with _ -> ())
 
 let file_print_ed2k_link filename filesize md4hash =
@@ -869,6 +981,7 @@ let new_file_ops network =
       op_file_commit = (fun _ _ -> ni_ok network "file_commit");
       op_file_save_as = (fun _ _ -> ni_ok network "file_save_as");
 (*    op_file_print = (fun _ _ -> ni_ok network "file_print"); *)
+      op_file_shared = (fun _ -> None);
       op_file_to_option = (fun _ -> fni network "file_to_option");
       op_file_cancel = (fun _ -> ni_ok network "file_cancel");
       op_file_info = (fun _ -> fni network "file_info");
@@ -881,9 +994,10 @@ let new_file_ops network =
       op_file_set_format = (fun _ -> fni network "file_set_format");
       op_file_all_sources = (fun _ -> fni network "file_all_sources");
       op_file_active_sources = (fun _ -> fni network "file_active_sources");
+      op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
       op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
-      op_file_print_html = (fun _ _ -> ni_ok network "file_print_html");
-      op_file_print_sources_html = (fun _ _ -> fni network "file_print_sources_html");
+      op_file_print = (fun _ _ -> ni_ok network "file_print_html");
+      op_file_print_sources = (fun _ _ -> fni network "file_print_sources");
       op_file_debug = (fun _ -> "");
       op_file_proposed_filenames = (fun impl -> []);
     }
@@ -906,6 +1020,8 @@ let check_file_implementations () =
         lprintf_nl "op_file_commit";
       if c.op_file_save_as == cc.op_file_save_as then
         lprintf_nl "op_file_save_as";
+      if c.op_file_shared == cc.op_file_shared then
+        lprintf_nl "op_file_shared";
       if c.op_file_cancel == cc.op_file_cancel then
         lprintf_nl "op_file_cancel";
       if c.op_file_pause == cc.op_file_pause then
@@ -924,10 +1040,10 @@ let check_file_implementations () =
         lprintf_nl "op_file_all_sources";
       if c.op_file_active_sources == cc.op_file_active_sources then
         lprintf_nl "op_file_active_sources";
-      if c.op_file_print_html == cc.op_file_print_html then
-        lprintf_nl "op_file_print_html";
-      if c.op_file_print_sources_html == cc.op_file_print_sources_html then
-        lprintf_nl "op_file_print_sources_html";
+      if c.op_file_print == cc.op_file_print then
+        lprintf_nl "op_file_print";
+      if c.op_file_print_sources == cc.op_file_print_sources then
+        lprintf_nl "op_file_print_sources";
   ) !files_ops;
   lprint_newline ()
 
@@ -949,12 +1065,6 @@ let _ =
       ) files_by_num;
       Printf.bprintf buf "  files: %d\n" !counter;
       Printf.bprintf buf "  files_ops: %d\n" (List.length !files_ops);
-      let counter = ref 0 in
-      HashMagic.iter (fun _ -> incr counter) files_magic;
-      Printf.bprintf buf "  files_magic: %d\n" !counter;
-      let counter = ref 0 in
-      HashComments.iter (fun _ -> incr counter) file_comments;
-      Printf.bprintf buf "  files_comments: %d\n" !counter
   )
 
 
@@ -1072,7 +1182,7 @@ let impl_file_info impl =
   let module T = GuiTypes in
   {
     T.file_fields = T.Fields_file_info.all;
-    T.file_comment = "";
+    T.file_comment = impl.impl_file_comment;
     T.file_name = impl.impl_file_best_name;
     T.file_names = impl.impl_file_filenames;
     T.file_num = impl.impl_file_num;
@@ -1095,4 +1205,14 @@ let impl_file_info impl =
     T.file_chunks_age = [||];
     T.file_uids = [];
     T.file_sub_files = [];
+    T.file_magic = impl.impl_file_magic;
+    T.file_comments = [];
+    T.file_user = impl.impl_file_owner.user_name;
+    T.file_group = user2_print_group impl.impl_file_group;
+    T.file_release = impl.impl_file_release;
   }
+
+let lprintf_file_nl file fmt =
+  lprintf_nl2 ("[" ^ (file_network file).network_shortname ^
+	       "] [file_num " ^ (string_of_int (file_num file)) ^ "]" ^
+	       "[temp " ^ (file_disk_name file) ^ "]") fmt
