@@ -18,10 +18,12 @@
 *)
 
 type 'a file_impl = {
+    mutable impl_file_owner : CommonTypes.userdb;
+    mutable impl_file_group : CommonTypes.groupdb option;
     mutable impl_file_update : int;
     mutable impl_file_state : CommonTypes.file_state;
     
-  mutable impl_file_comment : string list;
+    mutable impl_file_comment : string;
   mutable impl_file_num : int;
   mutable impl_file_val : 'a;
   mutable impl_file_ops : 'a file_ops;
@@ -36,6 +38,7 @@ type 'a file_impl = {
   mutable impl_file_filenames : string list;
   mutable impl_file_magic : string option;
   mutable impl_file_priority : int;
+  mutable impl_file_release : bool;
     mutable impl_file_last_seen : int;
     mutable impl_file_probable_name : string option;
 } 
@@ -43,6 +46,7 @@ and 'a file_ops = {
   mutable op_file_network : CommonTypes.network;
   mutable op_file_commit : 'a -> string -> unit;
   mutable op_file_save_as : 'a -> string -> unit;
+  mutable op_file_shared : 'a -> CommonTypes.shared option;
   mutable op_file_to_option : 'a -> (string * Options.option_value) list;
   mutable op_file_cancel : 'a -> unit;
   mutable op_file_pause : 'a -> unit;
@@ -53,9 +57,10 @@ and 'a file_ops = {
   mutable op_file_recover : 'a -> unit;
   mutable op_file_all_sources : 'a -> CommonTypes.client list;
   mutable op_file_active_sources : 'a -> CommonTypes.client list;
+  mutable op_file_comment : 'a -> string;
   mutable op_file_set_priority : 'a -> int -> unit;
-    mutable op_file_print_html: 'a -> Buffer.t -> unit;
-    mutable op_file_print_sources_html : 'a -> Buffer.t -> unit;
+    mutable op_file_print: 'a -> CommonTypes.ui_conn -> unit;
+    mutable op_file_print_sources : 'a -> CommonTypes.ui_conn -> unit;
     mutable op_file_files : ('a -> 'a file_impl -> CommonTypes.file list);    
     mutable op_file_debug : 'a -> string;
     mutable op_file_proposed_filenames : 'a -> string list;
@@ -75,10 +80,14 @@ val update_file_num : 'a file_impl -> unit
 val update_file_state : 'a file_impl -> CommonTypes.file_state -> unit
 val file_to_option : CommonTypes.file -> (string * Options.option_value) list
 val file_save_as : CommonTypes.file -> string -> unit
+val file_shared : CommonTypes.file -> CommonTypes.shared option
+val file_comment : CommonTypes.file -> string
 val file_network : CommonTypes.file -> CommonTypes.network
 val file_info : CommonTypes.file -> GuiTypes.file_info
-val file_pause : CommonTypes.file -> unit
-val file_resume : CommonTypes.file -> unit
+val file_pause : CommonTypes.file -> CommonTypes.userdb -> unit
+val file_resume : CommonTypes.file -> CommonTypes.userdb -> unit
+val set_file_release : CommonTypes.file -> bool -> CommonTypes.userdb -> unit
+val file_release : CommonTypes.file -> bool
 val set_file_state : CommonTypes.file -> CommonTypes.file_state -> unit
 val file_best_name : CommonTypes.file -> string
 val set_file_best_name : CommonTypes.file -> string -> string -> int -> unit
@@ -90,7 +99,7 @@ val file_recover : CommonTypes.file -> unit
 val file_preview : CommonTypes.file -> unit
 val file_all_sources : CommonTypes.file -> CommonTypes.client list
 val file_active_sources : CommonTypes.file -> CommonTypes.client list
-val file_print_sources_html : CommonTypes.file -> Buffer.t -> unit
+val file_print_sources : CommonTypes.file -> CommonTypes.ui_conn -> unit
 val files_ops : (int file_ops * int file_ops) list ref
 val new_file_ops : CommonTypes.network -> 'a file_ops
 val check_file_implementations : unit -> unit
@@ -118,8 +127,7 @@ val set_file_priority : CommonTypes.file -> int -> unit
 val set_file_last_seen : CommonTypes.file -> int -> unit
 val file_debug : CommonTypes.file -> string
 val set_file_comment : CommonTypes.file -> string -> unit
-val file_comment : CommonTypes.file -> string list
-val file_comment_length : CommonTypes.file -> int
+val file_comment : CommonTypes.file -> string
 val file_magic : CommonTypes.file -> string option
 val set_file_magic : CommonTypes.file -> string option -> unit
 val check_magic : CommonTypes.file -> unit  
@@ -136,3 +144,13 @@ val propose_filename : CommonTypes.file -> unit
 
 val forceable_download : CommonTypes.result_info list ref
 val impl_file_info : 'a file_impl -> GuiTypes.file_info
+
+val user2_filter_files : CommonTypes.file list -> CommonTypes.userdb -> CommonTypes.file list
+val user2_num_user_dls : CommonTypes.userdb -> int
+val user2_num_group_dls : CommonTypes.groupdb -> int
+val user2_allow_file_admin : CommonTypes.file -> CommonTypes.userdb -> bool
+val set_file_owner : CommonTypes.file -> CommonTypes.userdb -> unit
+val file_owner : CommonTypes.file -> CommonTypes.userdb
+val set_file_group : CommonTypes.file -> CommonTypes.groupdb option -> unit
+val file_group : CommonTypes.file -> CommonTypes.groupdb option
+val lprintf_file_nl : CommonTypes.file -> ('a, unit, unit) Pervasives.format -> 'a
