@@ -145,7 +145,8 @@ let _ =
   end;
 
   lprintf_nl "MLDonkey is working in %s" file_basedir;
-  if not (Sys.file_exists file_basedir) then begin
+  if not (Sys.file_exists file_basedir) ||
+	 not (Sys.file_exists (Filename.concat file_basedir "downloads.ini")) then begin
     lprint_newline ();
     lprintf_nl "creating new MLDonkey base directory in %s\n" file_basedir;
     created_new_base_directory := Some file_basedir
@@ -199,6 +200,11 @@ let _ =
       exit 2
     end;  
 
+  (* Charset conversion self-test *)
+  let filename = "abcdefghijklmnopqrstuvwxyz" in
+  let conv_filename = Charset.to_locale filename in
+  if filename <> conv_filename then Charset.conversion_enabled := false;
+
   Unix2.can_write_to_directory (Filename2.temp_directory ());
 
   if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet") then begin
@@ -333,9 +339,7 @@ let _ =
 
   Options.set_string_wrappers ip_range_list_option
     (fun list ->
-      List.fold_left (fun s ip ->
-	  Printf.sprintf "%s %s" (Ip.string_of_range ip) s
-      ) "" list
+      String.concat " " (List.map Ip.string_of_range (List.rev list))
   )
   (fun s ->
       let list = String2.tokens s in
@@ -357,6 +361,7 @@ let _ =
   String2.tokens
 
 let is_not_spam = ref (fun _ -> true)
+let is_not_comment_spam = ref (fun _ -> true)
 
 
 
@@ -578,6 +583,14 @@ let max_hard_download_rate = define_option current_section ["max_hard_download_r
   < 4      -> download limited to upload * 3"
     int_option 50
 
+let max_hard_upload_rate_2 = define_option current_section ["max_hard_upload_rate_2"]
+    "Second maximal upload rate for easy toggling (use bw_toggle)"
+    int_option 5
+
+let max_hard_download_rate_2 = define_option current_section ["max_hard_download_rate_2"]
+    "Second maximal download rate for easy toggling (use bw_toggle)"
+    int_option 20
+
 let max_opened_connections = define_option current_section ["max_opened_connections"]
   "Maximal number of opened connections"
     int_option 200
@@ -590,6 +603,10 @@ let max_upload_slots = define_option current_section ["max_upload_slots"]
   "How many slots can be used for upload"
     int_option 5
 
+let max_release_slots = define_option current_section ["max_release_slots"]
+  "How many percent of upload slots can be used for downloading files tagged as release"
+    percent_option 20
+
 let friends_upload_slot = define_option current_section ["friends_upload_slot"]
   "Set aside a single reserved slot to upload to friends"
     bool_option true
@@ -725,6 +742,14 @@ let html_mods_vd_comments = define_expert_option current_section ["html_mods_vd_
   "Whether to display the Comments column in vd output"
     bool_option true
 
+let html_mods_vd_user = define_expert_option current_section ["html_mods_vd_user"]
+  "Whether to display the User column in vd output"
+    bool_option false
+
+let html_mods_vd_group = define_expert_option current_section ["html_mods_vd_group"]
+  "Whether to display the Group column in vd output"
+    bool_option false
+
 let html_mods_vd_active_sources = define_expert_option current_section ["html_mods_vd_active_sources"]
   "Whether to display the Active Sources column in vd output"
     bool_option true
@@ -774,13 +799,30 @@ let html_mods_vd_gfx_h = define_expert_option current_section ["html_mods_vd_gfx
     bool_option true
 
 let html_mods_vd_gfx_x_size = define_expert_option current_section ["html_mods_vd_gfx_x_size"]
-  "Graph x size in vd output ( 320 < x < 3600 )"
-    int_option 630
+  "Graph x size in vd output ( 365 < x < 3665 )"
+    int_option 795
 
 let html_mods_vd_gfx_y_size = define_expert_option current_section ["html_mods_vd_gfx_y_size"]
   "Graph y size in vd output ( 200 < y < 1200 )"
     int_option 200
 
+let html_mods_vd_gfx_h_intervall = define_expert_option current_section ["html_mods_vd_gfx_h_intervall"]
+  "compute values for hourly graph every 1,2,3,4,5,10,15,20,30,60 min
+	Changes to this option require a core restart."
+     int_option 60
+
+let html_mods_vd_gfx_h_dynamic = define_expert_option current_section ["html_mods_vd_gfx_h_dymamic"]
+  "Dynamic grid width, start with 1 h/grid, maximum html_mods_vd_gfx_h_grid_time h/grid"
+    bool_option true
+		
+let html_mods_vd_gfx_h_grid_time = define_expert_option current_section ["html_mods_vd_gfx_h_grid_time"]
+  "Max hours on time scale per grid (0 = no limit)"
+    int_option 0
+
+let html_mods_vd_gfx_subgrid = define_expert_option current_section ["html_mods_vd_gfx_subgrid"]
+  "Number of shown subgrids on graph (0 = no subgrids)"
+    int_option 0
+
 let html_mods_vd_gfx_tag = define_expert_option current_section ["html_mods_vd_gfx_tag"]
   "Draw tag graph"
     bool_option false
@@ -986,6 +1028,10 @@ let rss_feeds = define_expert_option current_section ["rss_feeds"]
   "URLs of RSS feeds"
     (list_option Url.option) []
 
+let rss_preprocessor = define_expert_option current_section ["rss_preprocessor"]
+  "If MLDonkey can not read broken RSS feeds, use this program to preprocess them"
+    string_option "xmllint"
+
 let ip_blocking_descriptions = define_expert_option current_section ["ip_blocking_descriptions"]
   "Keep IP blocking ranges descriptions in memory"
     bool_option false
@@ -1288,10 +1334,6 @@ let allow_any_command = define_option current_section ["allow_any_command"]
   ones in allowed_commands"
     bool_option false
 
-let enable_user_config = define_option current_section ["enable_user_config"]
-  "Are all users allowed to change MLDonkey options?"
-    bool_option true
-
 let allow_browse_share = define_option current_section ["allow_browse_share"]
   "Allow others to browse our share list (0: none, 1: friends only, 2: everyone"
     allow_browse_share_option 1
@@ -1300,6 +1342,10 @@ let messages_filter = define_option current_section ["messages_filter"]
   "Regexp of messages to filter out, example: string1|string2|string3"
     string_option "Your client is connecting too fast"
 
+let comments_filter = define_option current_section ["comments_filter"]
+  "Regexp of comments to filter out, example: string1|string2|string3"
+    string_option "http://|https://|www\\."
+
 
 
 
@@ -1417,10 +1463,6 @@ let http_realm = define_expert_option current_section ["http_realm"]
   "The realm shown when connecting with a WEB browser"
     string_option "MLdonkey"
 
-let use_html_frames = define_expert_option current_section ["use_html_frames"]
-  "This option controls whether the WEB interface should use frames or not"
-    bool_option true
-
 let html_frame_border = define_expert_option current_section ["html_frame_border"]
   "This option controls whether the WEB interface should show frame borders or not"
     bool_option true
@@ -1516,7 +1558,7 @@ let calendar = define_expert_option current_section ["calendar"]
 
 let compaction_overhead = define_expert_option current_section ["compaction_overhead"]
   "The percentage of free memory before a compaction is triggered"
-    int_option 25
+    percent_option 25
 
 let space_overhead = define_expert_option current_section ["space_overhead"]
   "The major GC speed is computed from this parameter. This is the memory
@@ -1524,7 +1566,7 @@ let space_overhead = define_expert_option current_section ["space_overhead"]
   unreachable blocks. It is expressed as a percentage of the memory used
   for live data. The GC will work more (use more CPU time and collect 
   blocks more eagerly) if space_overhead is smaller."
-    int_option 80
+    percent_option 80
 
 let max_displayed_results = define_expert_option current_section ["max_displayed_results"]
   "Maximal number of results displayed for a search"
@@ -1535,6 +1577,13 @@ let options_version = define_expert_option current_section ["options_version"]
     int_option 14
 
 
+let max_comments_per_file = define_expert_option current_section ["max_comments_per_file"]
+  "Maximum number of comments per file"
+  int_option 100
+
+let max_comment_length = define_expert_option current_section ["max_comment_length"]
+  "Maximum length of file comments"
+  int_option 256
 
 
 (*************************************************************************)
@@ -1748,14 +1797,20 @@ let _ =
         (!!minor_heap_size * 1024) };
   );
   option_hook client_buffer_size (fun _ ->
-      TcpBufferedSocket.max_buffer_size := maxi 10000000 !!client_buffer_size
+      TcpBufferedSocket.max_buffer_size := max 10000000 !!client_buffer_size
   );
-  if Autoconf.has_gd then
+  if Autoconf.has_gd then begin
     option_hook html_mods_vd_gfx_png (fun _ ->
       if not Autoconf.has_gd_png && !!html_mods_vd_gfx_png then html_mods_vd_gfx_png =:= false;
       if not Autoconf.has_gd_jpg && not !!html_mods_vd_gfx_png then html_mods_vd_gfx_png =:= true
+	);
+	option_hook html_mods_vd_gfx_h_intervall (fun _ -> 
+	let values = [1; 2; 3; 4; 5; 10; 15; 20; 30; 60] in 
+	let v = List.find ((<=) (min !!html_mods_vd_gfx_h_intervall 60)) values in 
+	if v <> !!html_mods_vd_gfx_h_intervall then html_mods_vd_gfx_h_intervall =:= v 
     )
-
+	end
+	
 let verbose_msg_clients = ref false
 let verbose_msg_raw = ref false
 let verbose_msg_clienttags = ref false
@@ -1908,14 +1963,25 @@ let quote_unquote_bars m =
   in aux 0
       
 let _ =
-  option_hook messages_filter (fun _ ->
-      is_not_spam := if !!messages_filter <> "" then
-        let r = Str.regexp_case_fold (quote_unquote_bars !!messages_filter) in
-        (fun s -> try
+  let regex_fun str = 
+    if str <> "" then
+      let r = Str.regexp_case_fold (quote_unquote_bars str) in
+      (fun s -> 
+        try
               ignore (Str.search_forward r s 0);
               false
-            with Not_found -> true)
-      else (fun _ -> true))
+        with Not_found -> true
+      )
+    else (fun _ -> true)
+  in
+
+  option_hook messages_filter (fun _ ->
+      is_not_spam := regex_fun !!messages_filter
+  );
+
+  option_hook comments_filter (fun _ ->
+      is_not_comment_spam := regex_fun !!comments_filter
+  )
 
 let http_proxy = ref None
 
