@@ -53,6 +53,7 @@ and 'a server_ops = {
     mutable op_server_connect : ('a -> unit);
     mutable op_server_disconnect : ('a -> unit);
     mutable op_server_users : ('a -> user list);
+    mutable op_server_published : ('a -> file list);
     mutable op_server_query_users : ('a -> unit);
     mutable op_server_find_user : ('a -> string -> unit);
     mutable op_server_cid : ('a -> Ip.t);
@@ -108,12 +109,15 @@ let impl_server_info impl =
     T.server_users = None;
     T.server_banner = "";
     T.server_preferred = false;
+    T.server_master = false;
     T.server_version = "";
     T.server_max_users = 0L;
     T.server_soft_limit = 0L;
     T.server_hard_limit = 0L;
     T.server_lowid_users = 0L;
     T.server_ping = 0;
+    T.server_published_files = 0;
+    T.server_features = None;
   }
 
 let server_num s =
@@ -183,6 +187,10 @@ let server_users s =
   let s = as_server_impl s in
   s.impl_server_ops.op_server_users s.impl_server_val
 
+let server_published s =
+  let s = as_server_impl s in
+  s.impl_server_ops.op_server_published s.impl_server_val
+
 let server_cid s =
   let s = as_server_impl s in
   s.impl_server_ops.op_server_cid s.impl_server_val
@@ -212,6 +220,7 @@ let new_server_ops network =
       op_server_disconnect = (fun _ -> ni_ok network "server_disconnect");
       op_server_find_user = (fun _ -> fni network "find_user");
       op_server_query_users = (fun _ -> ni_ok network "query_users");
+      op_server_published = (fun _ -> fni network "published");
       op_server_users = (fun _ -> fni network "users");
       op_server_cid = (fun _ -> fni network "cid");
       op_server_low_id = (fun _ -> fni network "low_id");
@@ -362,24 +371,30 @@ let server_banner s o =
       info.G.server_banner
 
 let server_print_html_header buf ext =
+     if !!html_mods_use_js_tooltips then Printf.bprintf buf
+"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:
+-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
+
     html_mods_table_header buf "serversTable" (Printf.sprintf "servers%s" ext) ([
     ( "1", "srh", "Server number", "#" ) ;
     ( "0", "srh", "Connect|Disconnect", "C/D" ) ;
     ( "0", "srh", "Remove", "Rem" ) ;
     ( "0", "srh", "Preferred", "P" ) ;
+    ( "0", "srh", "Master servers", "M" ) ;
     ( "0", "srh", "[Hi]gh or [Lo]w ID", "ID" ) ;
     ( "0", "srh", "Network name", "Network" ) ;
     ( "0", "srh", "Connection status", "Status" ) ;
-    ] @ (if !Geoip.active then [( "0", "srh", "Country Code/Name", "CC" )] else []) @ [
     ( "0", "srh br", "IP address", "IP address" ) ;
+    ] @ (if !Geoip.active then [( "0", "srh br", "Country Code/Name", "CC" )] else []) @ [
     ( "1", "srh ar", "Number of connected users", "Users" ) ;
     ( "1", "srh ar br", "Max number of users", "MaxUsers" ) ;
     ( "1", "srh ar br", "LowID users", "LowID" ) ;
-    ( "1", "srh ar br", "Number of files indexed on server", "Files" ) ;
+    ( "1", "srh ar", "Number of files indexed on server", "Files" );
+    ( "1", "srh ar br", "Number of published files on server", "Publ" );
     ( "1", "srh ar", "Soft file limit", "Soft" ) ;
     ( "1", "srh ar br", "Hard file limit", "Hard" ) ;
     ( "0", "srh ar br", "Ping (ms)", "Ping" ) ;
-    ( "0", "srh", "Server version", "Version" ) ;
+    ( "0", "srh br", "Server version", "Version" ) ;
     ( "0", "srh", "Server name", "Name" ) ;
     ( "0", "srh", "Server details", "Details" ) ])
 
@@ -397,80 +412,80 @@ let server_print s o =
     let buf = o.conn_buf in
   
   if use_html_mods o then begin
-  let snum = (server_num s) in
+    let snum = (server_num s) in
+    let ip_port_string = 
+      Printf.sprintf "%s:%s%s" 
+      (Ip.string_of_addr info.G.server_addr)
+       (string_of_int info.G.server_port)
+       (if info.G.server_realport <> 0 
+          then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
+            else ""
+        )
+    in
 
-    Printf.bprintf buf "
-    \\<tr class=\\\"dl-%d\\\"\\>
-    \\<td class=\\\"srb\\\" %s \\>%d\\</td\\>
-    %s
-    %s
-    %s
-    \\<td class=\\\"sr\\\" %s\\</td\\>
-    \\<td class=\\\"sr\\\"\\>%s\\</td\\>
-    \\<td class=\\\"sr\\\"\\>%s\\</td\\>
-    %s
-    \\<td class=\\\"sr br\\\"\\>%s:%s\\</td\\>
-    \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%Ld\\</td\\>
-    \\<td class=\\\"sr ar br\\\"\\>%d\\</td\\>
-    \\<td class=\\\"sr br\\\"\\>%s\\</td\\>
-    \\<td class=\\\"sr\\\"\\>%s\\</td\\>
-    \\<td width=\\\"100%%\\\" class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>\n"
-    (html_mods_cntr ())
+    Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\""
+    (html_mods_cntr ());
+
+    (if !!html_mods_use_js_tooltips then
+       Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s %s<br>%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
+        info.G.server_name ip_port_string 
+	(match info.G.server_features with
+	| None -> ""
+	| Some f -> "server features: " ^ f)
+       !!html_mods_js_tooltips_wait
+       !!html_mods_js_tooltips_timeout
+       !!html_mods_js_tooltips_wait);
+
+    Printf.bprintf buf 
+" \\<td class=\\\"srb\\\" %s \\>%d\\</td\\> %s %s %s"
     (match impl.impl_server_state with
-        Connected _ -> Printf.sprintf "title=\\\"Server Banner\\\"
-            onMouseOver=\\\"mOvr(this);\\\"
-            onMouseOut=\\\"mOut(this);\\\"
-            onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" snum
+        Connected _ -> 
+            Printf.sprintf "title=\\\"Server Banner\\\"
+            onClick=\\\"location.href='submit?q=server_banner+%d'\\\"" 
+            snum
         | _ -> "")
     snum
       (
-        if server_blocked s && (match impl.impl_server_state with
-         NotConnected _ -> true
-             | _ -> false) then "\\<td class=\\\"srb\\\"\\>blk\\</td\\>" else
-        Printf.sprintf
-        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
-        onMouseOut=\\\"mOut(this);\\\" title=\\\"Connect|Disconnect\\\"
-        onClick=\\\"parent.fstatus.location.href='submit?q=%s+%d'\\\"\\>%s\\</TD\\>"
-        (match impl.impl_server_state with
-           NotConnected _ -> "c"
-         | _ -> "x")
+     let not_connected =
+        match impl.impl_server_state with
+        | ServerFull
+        | NotConnected _ -> true
+        | _ -> false 
+      in 
+      if server_blocked s && not_connected 
+        then "\\<td class=\\\"srb\\\"\\>blk\\</td\\>" 
+        else Printf.sprintf
+        "\\<td class=\\\"srb\\\" title=\\\"Connect|Disconnect\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=%s+%d'\\\"\\>%s\\</td\\>"
+              (if not_connected then "c" else "x")
         snum
-        (match impl.impl_server_state with
-           NotConnected _ -> "Conn"
-         | _ -> "Disc")
+              (if not_connected then "Conn" else "Disc")
       )
       (
         Printf.sprintf
-        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
-        onMouseOut=\\\"mOut(this);\\\" title=\\\"Remove server\\\"
-        onClick=\\\"parent.fstatus.location.href='submit?q=rem+%d'\\\"\\>Rem\\</TD\\>"
+        "\\<td class=\\\"srb\\\" title=\\\"Remove server\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=rem+%d'\\\"\\>Rem\\</td\\>"
       snum
       )
       (
         if info.G.server_preferred then begin
         Printf.sprintf
-        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
-        onMouseOut=\\\"mOut(this);\\\" title=\\\"Unset preferred\\\"
-        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+false+%s'\\\"\\>T\\</TD\\>"
+        "\\<td class=\\\"srb\\\" title=\\\"Unset preferred\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+false+%s'\\\"\\>P\\</td\\>"
         (Ip.string_of_addr info.G.server_addr)
         end else begin
         Printf.sprintf
-        "\\<TD class=\\\"srb\\\" onMouseOver=\\\"mOvr(this);\\\"
-        onMouseOut=\\\"mOut(this);\\\" title=\\\"Set preferred\\\"
-        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+true+%s'\\\"\\>F\\</TD\\>"
+        "\\<td class=\\\"srb\\\" title=\\\"Set preferred\\\"
+        onClick=\\\"parent.fstatus.location.href='submit?q=preferred+true+%s'\\\"\\>-\\</td\\>"
         (Ip.string_of_addr info.G.server_addr)
         end
-      )
-      (if n.network_name = "Donkey" then
-         begin
+      );
+
+      let id_title, id_text = 
+        match n.network_name with
+          "Donkey" -> begin
            match impl.impl_server_state with
-           | Connected _ ->
-               begin
+                Connected _ -> begin
                  let cid = (server_cid s) in
                  let (label,shortlabel,our_ip) =
                    if not (server_low_id s) then
@@ -478,49 +493,58 @@ let server_print s o =
                       (if !!set_client_ip <> cid then
                          Printf.sprintf "(clientIP: %s)"
                            (Ip.to_string !!set_client_ip)
-                       else ""
-                      )
-                     )
-                   else
-                     ("LowID","Lo","")
+                       else ""))
+                    else ("LowID","Lo","")
                  in
-                 Printf.sprintf
-                    "title=\\\"%s: %s = %s %s\\\" \\>%s"
+                   Printf.sprintf "%s: %s = %s %s"
                       label
                       (Int64.to_string (Ip.to_int64 (Ip.rev cid)))
                       (Ip.to_string cid)
                       our_ip
-                      shortlabel
+                 ,shortlabel
                end
-           | _ -> "\\>"
+               | _ -> "",""
          end
-       else "\\>"
-      )
-      n.network_name
-      (match impl.impl_server_state with
-        NotConnected _ -> if server_blocked s then "IP blocked"
-        else (string_of_connection_state impl.impl_server_state)
-      | _ -> (string_of_connection_state impl.impl_server_state))
-      (if !Geoip.active then 
-	 Printf.sprintf "\\<td class=\\\"sr\\\" title=\\\"%s\\\" \\>%s\\</td\\>" cn cc
-       else "")
-      (Ip.string_of_addr info.G.server_addr)
-      (Printf.sprintf "%s%s"
-       (string_of_int info.G.server_port)
-       (if info.G.server_realport <> 0 
-          then "(" ^ (string_of_int info.G.server_realport) ^ ")" 
-          else ""))
-      info.G.server_nusers
-      info.G.server_max_users
-      info.G.server_lowid_users
-      info.G.server_nfiles
-      info.G.server_soft_limit
-      info.G.server_hard_limit
-      info.G.server_ping
-      info.G.server_version
-      info.G.server_name
-      info.G.server_description
+      | _ -> "",""
+    in
 
+    let server_state_string = 
+      match impl.impl_server_state with
+        NotConnected _ when server_blocked s -> "IP blocked"
+      | _ -> string_of_connection_state impl.impl_server_state
+    in
+
+    let cc,cn = Geoip.get_country (Ip.ip_of_addr info.G.server_addr) in
+    html_mods_td buf ([
+      ("", "srb", if info.G.server_master then "M" else "-");
+      (id_title, "sr", id_text);
+      ("", "sr", n.network_name);
+      ("", "sr", server_state_string);
+      ("", "sr br", ip_port_string);
+      ] @ (if !Geoip.active then [(cn, "sr br", cc)] else []) @ [
+      ("", "sr ar", if info.G.server_nusers = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_nusers);
+      ("", "sr ar br", if info.G.server_max_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_max_users);
+      ("", "sr ar br", if info.G.server_lowid_users = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_lowid_users);
+      ("", "sr ar", if info.G.server_nfiles = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_nfiles)]);
+
+    if info.G.server_published_files = 0 then
+      html_mods_td buf ([("", "sr br", "")])
+    else
+      Printf.bprintf buf
+"\\<TD class=\\\"sr br\\\" title=\\\"Show published files\\\"
+onClick=\\\"location.href='submit?q=server_shares+%d'\\\"\\>%d\\</TD\\>"
+	snum info.G.server_published_files;
+
+    html_mods_td buf ([
+      ("", "sr ar", if info.G.server_soft_limit = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_soft_limit);
+      ("", "sr ar br", if info.G.server_hard_limit = Int64.zero then "" else Printf.sprintf "%Ld" info.G.server_hard_limit);
+      ("", "sr ar br", if info.G.server_ping = 0 then "" else Printf.sprintf "%d" info.G.server_ping);
+      ("", "sr br", info.G.server_version);
+      ("", "sr", info.G.server_name);
+    ]);
+
+    Printf.bprintf buf "\\<td width=\\\"100%%\\\" class=\\\"sr\\\"\\>%s\\</td\\>\\</tr\\>\n"
+      info.G.server_description;
   end
    else
   begin
