@@ -43,7 +43,8 @@ type 'a shared_impl = {
     mutable impl_shared_size : int64;
     mutable impl_shared_id : Md4.t;
     mutable impl_shared_requests : int;
-    mutable impl_shared_magic : string option
+    mutable impl_shared_magic : string option;
+    mutable impl_shared_servers : CommonTypes.server list;
   }
   
 and 'a shared_ops = {
@@ -214,6 +215,7 @@ let dummy_shared = {
     impl_shared_id = Md4.null;
     impl_shared_requests = 0;
     impl_shared_magic = None;
+    impl_shared_servers = []
   }
   
   
@@ -235,17 +237,17 @@ let shared_add_directory shared_dir local_dir =
   
 let shared_scan_directory shared_dir local_dir =
   let incoming_files_inode = 
-    ((Unix.stat ((CommonComplexOptions.incoming_files ()).shdir_dirname)).Unix.st_ino)
+    ((Unix.stat ((CommonComplexOptions.incoming_dir false ()).shdir_dirname)).Unix.st_ino)
   in
   let incoming_directories_inode = 
-    ((Unix.stat ((CommonComplexOptions.incoming_directories ()).shdir_dirname)).Unix.st_ino)
+    ((Unix.stat ((CommonComplexOptions.incoming_dir true ()).shdir_dirname)).Unix.st_ino)
   in
   let temp_directory_inode =
     ((Unix.stat !!temp_directory).Unix.st_ino)
   in
   let dirname = shared_dir.shdir_dirname in
   let strategy = 
-    CommonComplexOptions.sharing_strategies shared_dir.shdir_strategy in
+    CommonComplexOptions.sharing_strategy shared_dir.shdir_strategy in
   let dirname = 
     if Filename.is_relative dirname then
       Filename.concat file_basedir dirname
@@ -352,6 +354,7 @@ let impl_shared_info impl =
     T.shared_requests = impl.impl_shared_requests; 
     T.shared_uids = [];
     T.shared_sub_files = [];
+    T.shared_magic = impl.impl_shared_magic;
   }
   
 let shared_info s =
