@@ -506,13 +506,29 @@ let get_int_date proto s pos =
 let get_int_pos s pos =
   get_int s pos, pos + 4
   
-let get_sub_files s pos =
+let get_sub_files proto s pos =
   get_list (fun s pos ->
     let name, pos = get_string s pos in
     let size, pos = get_int64 s pos, pos+8 in
-    (name, size, Some ""), pos
+    let magic, pos = 
+      if proto > 40 then
+        get_string s pos
+      else
+        "", pos
+    in
+    (name, size, Some magic), pos
   ) s pos
   
+let get_file_comments proto s pos = 
+  get_list (fun s pos ->
+     let ip, pos = get_ip2 proto s pos in
+     let name, pos = get_string s pos in
+     let rating, pos = get_uint8 s pos, pos+1 in
+     let comment, pos = get_string s pos in
+     (ip, name, rating, comment), pos
+  ) s pos
+
+  
 let get_file proto s pos = 
   let num = get_int s pos in
   let net = get_int s (pos+4) in
@@ -557,10 +573,31 @@ let get_file proto s pos =
   in
   let sub_files, pos =
     if proto > 35 then
-      get_sub_files s pos
+      get_sub_files proto s pos
     else [], pos
   in
-
+  let magic, pos = 
+    if proto > 40 then
+      let ms, pos = get_string s pos
+      in Some ms, pos 
+    else
+      Some "", pos
+  in
+  let comments, pos = 
+    if proto > 40 then
+      get_file_comments proto s pos
+    else [], pos
+  in
+  let user, pos = 
+    if proto > 40 
+      then get_string s pos
+      else "", pos
+  in
+  let group, pos = 
+    if proto > 40 
+      then get_string s pos
+      else "", pos
+  in
   (*
   assert (num = file_info_test.file_num);
   assert (net = file_info_test.file_network);
@@ -609,6 +646,11 @@ let get_file proto s pos =
     file_priority = priority;
     file_uids = uids;
     file_sub_files = sub_files;
+    file_magic = magic;
+    file_comments = comments;
+    file_user = user;
+    file_group = group;
+    file_release = false;
   }, pos
 
 let get_host_state proto s pos =
@@ -709,12 +751,15 @@ let get_server proto s pos =
     server_banner = "";
     server_users = None;
     server_preferred = preferred;
+    server_master = false;
     server_version = ve; 
     server_max_users = ma;
     server_lowid_users = lo;
     server_soft_limit = so;
     server_hard_limit = ha;
     server_ping = pi;
+    server_published_files = 0;
+    server_features = None;
   }, pos
 
 let get_client_type s pos = 
@@ -768,6 +813,7 @@ let get_client proto s pos =
       client_files = None;
       client_connect_time = 0;
       client_software = "";
+      client_os = None;
       client_release = "";
       client_emulemod = "";
       client_downloaded = zero;
@@ -826,6 +872,7 @@ let get_client proto s pos =
     client_files = None;
     client_connect_time = connect_time;
     client_software = software;
+    client_os = None;
     client_release = release;
     client_emulemod = emulemod;
     client_downloaded = downloaded;
@@ -878,7 +925,7 @@ let get_network proto s pos =
     network_config_filename = config_file;
     network_uploaded = uploaded;
     network_downloaded = downloaded;
-    network_connected = connected;
+    network_connected_servers = connected;
   }, pos
 
 
@@ -940,6 +987,7 @@ let get_shared_info proto s pos =
     shared_requests = requests;
     shared_uids = [];
     shared_sub_files = [];
+    shared_magic = Some "";
   }
 
 let get_shared_info_version_10 proto s pos =
@@ -957,9 +1005,16 @@ let get_shared_info_version_10 proto s pos =
   in
   let sub_files, pos =
     if proto > 36 then
-      get_sub_files s pos
+      get_sub_files proto s pos
     else [], pos
   in
+  let magic, pos = 
+    if proto > 40 then
+      let ms, pos = get_string s (pos) in
+      Some ms, pos
+    else
+      Some "", pos
+  in
   {
     shared_num = num;
     shared_network = network;
@@ -969,6 +1024,7 @@ let get_shared_info_version_10 proto s pos =
     shared_requests = requests;
     shared_uids = uids;
     shared_sub_files = sub_files;
+    shared_magic = magic;
   }
 
 
@@ -998,7 +1054,7 @@ let from_gui (proto : int array) opcode s =
     | 5
     | 52 ->
         if proto < 14 then
-           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user, pass)
+           let pass = fst (get_string s 2) in Password (CommonUserDb.admin_user.CommonTypes.user_name, pass)
         else
         let pass,pos = get_string s 2 in
         let login,pos = get_string s pos in
@@ -1262,6 +1318,10 @@ let from_gui (proto : int array) opcode s =
          let b = get_bool s 6 in
          ServerSetPreferred (num, b)
 
+    | 68 ->
+         let num = get_int s 2 in
+         GetStats num
+
     | _ -> 
         lprintf_nl "FROM GUI:Unknown message %d" opcode; 
         raise FromGuiMessageNotImplemented
