@@ -136,6 +136,11 @@ type file_info = {
     mutable file_priority : int;
     mutable file_uids : Uid.t list;
     mutable file_sub_files : (string * int64 * string option) list;
+    mutable file_magic : string option;
+    mutable file_comments : (Ip.t * string * int * string) list;
+    mutable file_user : string;
+    mutable file_group : string;
+    mutable file_release : bool;
   }
   
 type user_info = {
@@ -165,12 +170,15 @@ type server_info = {
     mutable server_users : int list option;
     mutable server_banner : string;
     mutable server_preferred : bool;
+    mutable server_master : bool;
     mutable server_version : string;
     mutable server_max_users : int64;
     mutable server_soft_limit : int64;
     mutable server_hard_limit : int64;
     mutable server_lowid_users : int64;
     mutable server_ping : int;
+    mutable server_published_files : int;
+    mutable server_features : string option;
 
   } 
 
@@ -208,6 +216,7 @@ type client_info = {
     mutable client_chat_port : int;
     mutable client_connect_time : int;
     mutable client_software : string;
+    mutable client_os : string option;
     mutable client_release : string;
     mutable client_emulemod : string;
     mutable client_downloaded : int64;
@@ -240,9 +249,32 @@ type shared_info = {
     mutable shared_requests : int;
     mutable shared_uids : Uid.t list; (* net file UID *)
     mutable shared_sub_files : (string * int64 * string option) list;
+    mutable shared_magic : string option;
   }
   
   
+let osinfo_short i =
+  match i with
+    Some s when
+      s = "linux" ||
+      s = "netbsd" ||
+      s = "macos" ||
+      s = "freebsd" ||
+      s = "windows" -> Some (String.sub s 0 1)
+  | _ -> i
+
+let client_software_short software os =
+  software ^
+  match osinfo_short os with
+    Some s -> "/" ^ s
+  | None -> ""
+
+let client_software software os =
+  software ^
+  match os with
+    Some s -> "/" ^ s
+  | None -> ""
+
 let add_file tree dirname r =
   let path = Filename2.path_of_filename dirname in
   
@@ -326,5 +358,10 @@ let file_info_test =
     file_priority = 0;
    file_uids = [];
     file_sub_files = [];
+    file_magic = Some "";
+    file_comments = [];
+    file_user = "";
+    file_group = "";
+    file_release = false;
  } 
   
