@@ -310,7 +310,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
         let user, pass =
           match args with
             [] -> failwith "Usage: auth <user> <password>"
-          | [s1] -> admin_user, s1
+          | [s1] -> admin_user.CommonTypes.user_name, s1
           | user :: pass :: _ -> user, pass
         in
         if valid_password user pass then begin
@@ -320,7 +320,7 @@ Use '$rhelp command$n' or '$r? command$n' for help on a command.
             let module M = CommonMessages in
             Buffer.add_string buf M.full_access;
             (match DriverInteractive.real_startup_message () with
-               Some s -> Buffer.add_string buf s;
+               Some s -> Buffer.add_string buf ("\n" ^ s);
              | None -> ());
           end else
         let module M = CommonMessages in
@@ -581,7 +581,7 @@ let telnet_handler t event =
           "telnet connection"
           s in
         let telnet = {
-            telnet_auth = ref (empty_password admin_user);
+            telnet_auth = ref (has_empty_password admin_user);
             telnet_iac = false;
             telnet_wait = 0;
             telnet_buffer = Buffer.create 100;
@@ -775,14 +775,6 @@ let read_theme_page page =
     really_input file s 0 size;
     s)
 
-let add_simple_commands buf =
-  let this_page = "commands.html" in
-  Buffer.add_string buf (
-    if !!html_mods_theme != "" && theme_page_exists this_page then
-      read_theme_page this_page else
-    if !!html_mods then !!CommonMessages.web_common_header_mods0
-    else !!CommonMessages.web_common_header_old)
-
 let http_add_gen_header r =
   add_reply_header r "Server" "MLdonkey";
   add_reply_header r "Connection" "close"
@@ -883,9 +875,7 @@ let html_open_page buf t r open_body =
                         else !!CommonMessages.html_header_old);
 
   Buffer.add_string buf "</head>\n";
-  if open_body then Buffer.add_string buf "<body>\n";
-  if not !!use_html_frames then add_simple_commands buf;
-  ()
+  if open_body then Buffer.add_string buf "<body>\n"
 
 let html_close_page buf close_body =
   if close_body then Buffer.add_string buf "</body>\n";
@@ -955,12 +945,18 @@ let http_handler o t r =
 	 List.iter (fun (arg, value) -> Printf.bprintf b " %s %s" arg value) r.get_url.Url.args;
 	 if Buffer.contents b <> "" then Printf.sprintf "(%s)" (Buffer.contents b) else "");
 
-  let user = if r.options.login = "" then admin_user else r.options.login in
+  let user = if r.options.login = "" then admin_user.CommonTypes.user_name else r.options.login in
   if not (valid_password user r.options.passwd) then begin
       clear_page buf;
       http_file_type := HTM;
-      Buffer.add_string buf (snd(Http_server.error_page "401" "" "" (Ip.to_string (TcpBufferedSocket.my_ip r.sock)) (string_of_int !!http_port) None));
-      need_auth r !!http_realm
+      let _, error_text_long, header = Http_server.error_page "401" "" ""
+	(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
+	(string_of_int !!http_port) None in
+      Buffer.add_string buf error_text_long;
+      r.reply_head <- header;
+      r.reply_headers <- [
+        "Connection", "close";
+        "WWW-Authenticate", Printf.sprintf "Basic realm=\"%s\"" !!http_realm]
     end
   else
     begin
@@ -1000,15 +996,15 @@ let http_handler o t r =
                     "VDC" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_cancel file
+                      file_cancel file o.conn_user.ui_user
                   | "VDP" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_pause file
+                      file_pause file o.conn_user.ui_user
                   | "VDR" ->
                       let num = int_of_string value in
                       let file = file_find num in
-                      file_resume file
+                      file_resume file o.conn_user.ui_user
                   | _ -> ()
               ) r.get_url.Url.args;
 
@@ -1048,7 +1044,6 @@ let http_handler o t r =
               if !!html_mods then !!CommonMessages.multidllink_mods0
               else !!CommonMessages.multidllink_old)
         | "" | "index.html" ->
-            if !!use_html_frames then begin
                 html_open_page buf t r false;
                 let this_page = "frames.html" in
                 if !!html_mods_theme != "" && theme_page_exists this_page then
@@ -1079,9 +1074,7 @@ let http_handler o t r =
 </frameset>
 <frame name=\"output\" src=\"oneframe.html\">
 </frameset>
-" !!commands_frame_height;
-              end else
-              html_open_page buf t r true
+" !!commands_frame_height
         | "complex_search.html" ->
             html_open_page buf t r true;
             CommonSearch.complex_search buf
@@ -1093,7 +1086,7 @@ let http_handler o t r =
             Buffer.add_string buf (Printf.sprintf "<br><div align=\"center\"><h3>%s %s</h3></div>"
 	      (Printf.sprintf (_b "Welcome to MLDonkey")) Autoconf.current_version);
 	    if !!motd_html <> "" then Buffer.add_string buf !!motd_html;
-	    if user2_is_admin o.conn_user.ui_user_name then
+	    if user2_is_admin o.conn_user.ui_user then
 	    (match DriverInteractive.real_startup_message () with
 	       Some s -> Buffer.add_string buf (Printf.sprintf "<p><pre><b><h3>%s</b></h3></pre>" s);
 	     | None -> ())
@@ -1265,7 +1258,7 @@ let http_handler o t r =
                       try
                         let num = int_of_string value in
                         let r = find_result num in
-                        let files = result_download r [] false in
+                        let files = result_download r [] false o.conn_user.ui_user in
                         List.iter CommonInteractive.start_download files;
 
                         let module M = CommonMessages in
@@ -1285,15 +1278,23 @@ let http_handler o t r =
                   "cancel" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_cancel file
+                    file_cancel file o.conn_user.ui_user
                 | "pause" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_pause file
+                    file_pause file o.conn_user.ui_user
                 | "resume" ->
                     let num = int_of_string value in
                     let file = file_find num in
-                    file_resume file
+                    file_resume file o.conn_user.ui_user
+                | "release" ->
+                    let num = int_of_string value in
+                    let file = file_find num in
+                    set_file_release file true o.conn_user.ui_user
+                | "norelease" ->
+                    let num = int_of_string value in
+                    let file = file_find num in
+                    set_file_release file false o.conn_user.ui_user
                 | "sortby" ->
                     begin
                       match value with
@@ -1313,13 +1314,15 @@ let http_handler o t r =
                       | "N" -> o.conn_sortvd <- ByNet
                       | "Avail" -> o.conn_sortvd <- ByAvail
                       | "Cm" -> o.conn_sortvd <- ByComments
+                      | "User" -> o.conn_sortvd <- ByUser
+                      | "Group" -> o.conn_sortvd <- ByGroup
                       | _ -> ()
                     end
                 | _ -> ()
             ) r.get_url.Url.args;
             let b = Buffer.create 10000 in
 
-            let list = (List2.tail_map file_info !!files) in
+            let list = List2.tail_map file_info (user2_filter_files !!files o.conn_user.ui_user) in
             DriverInteractive.display_file_list b o list;
             html_open_page buf t r true;
             Buffer.add_string buf (html_escaped (Buffer.contents b))
@@ -1331,9 +1334,10 @@ let http_handler o t r =
                 | [ "jvcmd", "multidllink" ; "links", links] ->
                     html_open_page buf t r true;
                     List.iter (fun url ->
-		      if url <> "\013" && url <> "" then
+		      let url = fst (String2.cut_at url '\013') in
+		      if url <> "" then
 		        begin
-                          Buffer.add_string buf (html_escaped (dllink_parse (o.conn_output = HTML) url));
+                          Buffer.add_string buf (html_escaped (dllink_parse (o.conn_output = HTML) url o.conn_user.ui_user));
                           Buffer.add_string buf (html_escaped "\\<P\\>")
 			end
                     ) (String2.split links '\n')
@@ -1388,7 +1392,7 @@ let http_handler o t r =
 
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
-		  if (o.conn_user == default_user) || !!enable_user_config then
+		  if user2_is_admin o.conn_user.ui_user then
 		    begin
                       CommonInteractive.set_fully_qualified_options name value;
                       Buffer.add_string buf "Option value changed"
@@ -1489,11 +1493,34 @@ let http_handler o t r =
                 read_theme_page this_page else
               if !!html_mods then !!CommonMessages.download_html_js_mods0
               else !!CommonMessages.download_html_js_old)
-        | cmd ->
-            html_open_page buf t r true;
-            Printf.bprintf buf "No page named %s" (html_escaped cmd)
+        | "porttest" ->
+	    html_open_page buf t r true;
+	    let age time =
+	      Date.time_to_string (BasicSocket.last_time () - time) "verbose" in
+	    networks_iter (fun n -> 
+	      let result =
+		match network_porttest_result n with
+		  PorttestNotAvailable -> None
+	        | PorttestNotStarted -> Some "porttest not started"
+	        | PorttestInProgress time ->
+		    Some (Printf.sprintf "porttest started %s ago" (age time))
+	        | PorttestResult (time, s) ->
+		    Some (Printf.sprintf "porttest finished %s ago, %s" (age time) s)
+	      in
+	      (match result with
+		None -> ()
+	      | Some result ->
+		    Printf.bprintf buf "%s:<br> %s<br>\n" n.network_name result));
+		    Printf.bprintf buf "<br><br><a href=\"porttest\">Reload</a>"
+        | _ -> raise Not_found
       with
-      | Not_found -> Printf.bprintf buf "404 Not found"
+      | Not_found ->
+	  let _, error_text_long, header = Http_server.error_page "404" "" ""
+			(Ip.to_string (TcpBufferedSocket.my_ip r.sock))
+			(string_of_int !!http_port)
+			(Some (Url_not_found r.get_url.Url.full_file)) in
+	  r.reply_head <- header;
+	  Buffer.add_string buf error_text_long
       | e ->
           Printf.bprintf buf "\nException %s\n" (Printexc2.to_string e);
           r.reply_stream <- None
@@ -1501,11 +1528,11 @@ let http_handler o t r =
 
   let s =
     match !http_file_type with
-      HTM -> html_close_page buf false; dollar_escape o !!use_html_frames (Buffer.contents buf)
-    | MLHTM -> html_close_page buf true; dollar_escape o !!use_html_frames (Buffer.contents buf)
+      HTM -> html_close_page buf false; dollar_escape o true (Buffer.contents buf)
+    | MLHTM -> html_close_page buf true; dollar_escape o true (Buffer.contents buf)
     | TXT
+    | UNK
     | BIN -> Buffer.contents buf
-    | UNK -> "Unknown type for content :" ^ (Buffer.contents buf)
   in
   r.reply_content <- 
     if !http_file_type <> BIN && !!html_use_gzip then 
