@@ -246,6 +246,8 @@ let send_event gui ev =
     
 let send_update_file gui file_num update =
   let file = file_find file_num in
+  if user2_can_view_file gui.gui_conn.conn_user.ui_user (file_owner file) (file_group file) then
+  begin
   let impl = as_file_impl file in
   let file_info = if update then
       P.File_info (file_info file) 
@@ -256,6 +258,7 @@ let send_update_file gui file_num update =
         impl.impl_file_last_seen)
   in
   gui_send gui file_info
+  end
 
 let send_update_user gui user_num update =
   let user = user_find user_num in
@@ -428,7 +431,7 @@ let gui_initialize gui =
                   (File_add_source_event (file,c))
                 :: gui.gui_events.gui_new_events
             ) sources
-      ) !!files;
+      ) (user2_filter_files !!files gui.gui_conn.conn_user.ui_user);
       
       List.iter (fun file ->
           addevent gui.gui_events.gui_files (file_num file) true;
@@ -437,7 +440,7 @@ let gui_initialize gui =
       networks_iter_all (fun n ->
           List.iter (fun s ->
               addevent gui.gui_events.gui_servers (server_num s) true
-          ) (n.op_network_connected_servers ())
+          ) (network_connected_servers n)
       );
       
       server_iter (fun s ->
@@ -458,6 +461,7 @@ let gui_initialize gui =
             end
       );
       
+      if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
       shared_iter (fun s ->
           addevent gui.gui_events.gui_shared_files (shared_num s) true
       );
@@ -466,8 +470,11 @@ let gui_initialize gui =
           gui.gui_events.gui_new_events <- ev :: gui.gui_events.gui_new_events
       ) console_messages;                
       
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       gui_send gui (
         P.Options_info (simple_options "" downloads_ini));
+
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       networks_iter_all (fun r ->
           List.iter (fun opfile ->
               let prefix = r.network_shortname ^ "-" in
@@ -475,6 +482,7 @@ let gui_initialize gui =
               gui_send gui (P.Options_info args)) r.network_config_file);
 
 (* Options panels defined in downloads.ini *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       List.iter (fun s ->
           let section = section_name s in
           List.iter (fun o ->
@@ -484,6 +492,7 @@ let gui_initialize gui =
       ) (sections downloads_ini);
 
 (* Options panels defined in users.ini *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       List.iter (fun s ->
           let section = section_name s in
           List.iter (fun o ->
@@ -493,6 +502,7 @@ let gui_initialize gui =
       ) (sections users_ini);
 
 (* Options panels defined in each plugin *)
+      if user2_is_admin gui.gui_conn.conn_user.ui_user then
       networks_iter_all (fun r ->
           let prefix = r.network_shortname ^ "-" in
           List.iter (fun file ->
@@ -554,7 +564,7 @@ let gui_reader (gui: gui_record) t _ =
                     (File_add_source_event (file,c))
                     :: gui.gui_events.gui_new_events
                 ) (file_active_sources file)
-            ) !!files;
+            ) (user2_filter_files !!files gui.gui_conn.conn_user.ui_user);
             
           end
     
@@ -613,7 +623,7 @@ let gui_reader (gui: gui_record) t _ =
               ) list
           
           | P.SetOption (name, value) ->
-	      if user2_is_admin gui.gui_conn.conn_user.ui_user_name || !!enable_user_config then
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
 		CommonInteractive.set_fully_qualified_options name value
 	      else
 	        begin
@@ -644,8 +654,9 @@ let gui_reader (gui: gui_record) t _ =
               end
           
           | P.EnableNetwork (num, bool) ->
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
               let n = network_find_by_num num in
-              if n.op_network_is_enabled () <> bool then
+              if network_is_enabled n <> bool then
                 (try
                     if bool then network_enable n else network_disable n;
                   with e ->
@@ -670,7 +681,7 @@ let gui_reader (gui: gui_record) t _ =
                     network_extend_search r s e)
           
           | P.KillServer -> 
-	      if user2_is_admin gui.gui_conn.conn_user.ui_user_name then
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
 		CommonInteractive.clean_exit 0
 	      else
 	        begin
@@ -689,7 +700,7 @@ let gui_reader (gui: gui_record) t _ =
               add_timer 60. (fun _ ->
                   gui_send gui (Search_waiting (search_num, 0))
               );
-              gui.gui_id_counter <- maxi gui.gui_id_counter search_num;
+              gui.gui_id_counter <- max gui.gui_id_counter search_num;
               
               let user = gui.gui_conn.conn_user in
               let query = 
@@ -714,13 +725,11 @@ let gui_reader (gui: gui_record) t _ =
               networks_iter (fun r -> 
                   if search.search_network = 0 ||
                     r.network_num = search.search_network
-                  then
-                    
-                    r.op_network_search search buf);
+                  then network_search r search buf);
           
           | P.Download_query (filenames, num, force) ->
               let r = find_result num in
-              let files = result_download r filenames force in
+              let files = result_download r filenames force gui.gui_conn.conn_user.ui_user in
               List.iter CommonInteractive.start_download files
           
           | P.ConnectMore_query ->
@@ -731,7 +740,7 @@ let gui_reader (gui: gui_record) t _ =
                 if not (networks_iter_until_true
                     (fun n ->
                        try
-                         let s,r = network_parse_url n url in r
+                         let s,r = network_parse_url n url gui.gui_conn.conn_user.ui_user in r
                        with e ->
                          lprintf "Exception %s for network %s\n"
                            (Printexc2.to_string e) (n.network_name);
@@ -765,11 +774,13 @@ let gui_reader (gui: gui_record) t _ =
                 query_networks url
           
           | P.GetUploaders ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               gui_send gui (P.Uploaders
                   (List2.tail_map (fun c -> client_num c) 
                   (Intmap.to_list !uploaders)))
           
           | P.GetPending ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               gui_send gui (P.Pending (
                   List2.tail_map (fun c -> client_num c)
                   (Intmap.to_list !CommonUploads.pending_slots_map)))
@@ -778,13 +789,13 @@ let gui_reader (gui: gui_record) t _ =
               server_remove (server_find num)
           
           | P.SaveOptions_query list ->
-              
+	      if user2_is_admin gui.gui_conn.conn_user.ui_user then
               List.iter (fun (name, value) ->
                   CommonInteractive.set_fully_qualified_options name value) list;
               DriverInteractive.save_config ()
           
           | P.RemoveDownload_query num ->
-              file_cancel (file_find num)
+              file_cancel (file_find num) gui.gui_conn.conn_user.ui_user
           
           | P.ViewUsers num -> 
               let s = server_find num in
@@ -868,6 +879,7 @@ let gui_reader (gui: gui_record) t _ =
               client_connect c
           
           | P.DisconnectClient num ->
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               let c = client_find num in
               client_disconnect c
           
@@ -905,9 +917,9 @@ let gui_reader (gui: gui_record) t _ =
           | P.SwitchDownload (num, resume) ->
               let file = file_find num in
               if resume then
-                file_resume file          
+                file_resume file gui.gui_conn.conn_user.ui_user
               else
-                file_pause file
+                file_pause file gui.gui_conn.conn_user.ui_user
           
           | P.FindFriend user -> 
               networks_iter (fun n ->
@@ -1026,17 +1038,17 @@ let gui_reader (gui: gui_record) t _ =
   
           | NetworkMessage (num, s) ->
               let n = network_find_by_num num in
-              n.op_network_gui_message s
+              network_gui_message n s gui.gui_conn.conn_user.ui_user
                 
           | AddServer_query (num, ip, port) ->
               let n = network_find_by_num num in
 (* [CONTRIBUTE:] change the GUI protocol to transfer an address (ip/name)
   instead of just an ip. *)
               
-              let s = n.op_network_add_server (Ip.addr_of_ip ip) port in
+              let s = network_add_server n (Ip.addr_of_ip ip) port in
               server_connect s
           | RefreshUploadStats ->
-              
+              if user2_can_view_uploads gui.gui_conn.conn_user.ui_user then
               shared_iter (fun s ->
                   update_shared_info s;
               ) 
@@ -1044,6 +1056,11 @@ let gui_reader (gui: gui_record) t _ =
           | P.GetVersion ->
               gui_send gui (P.Version Autoconf.current_version)
 
+          | P.GetStats num ->
+              let n = network_find_by_num num in
+              let l = network_stat_info_list n in
+              gui_send gui (P.Stats (num, l))
+
           | P.GiftAttach (profile, version, client) ->
 	      let user, pass =
 		try
@@ -1080,13 +1097,13 @@ let gui_reader (gui: gui_record) t _ =
   with 
     Failure s ->
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
+  | Torrent_can_not_be_used ->
+      gui_send gui (Console (Printf.sprintf "\nError: This torrent does not have valid tracker URLs\n"))
+  | Torrent_already_exists ->
+      gui_send gui (Console (Printf.sprintf "\nError: This torrent is already in download queue\n"))
   | e ->
-      let error_text = Printexc2.to_string e in
-      if error_text = "BTInteractive.Torrent_can_not_be_used" then
-	gui_send gui (Console (Printf.sprintf "\nError: This torrent does not have valid tracker URLs\n"))
-      else
-	gui_send gui (Console (Printf.sprintf "from_gui: exception %s for message %s\n"
-          (Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
+      gui_send gui (Console (Printf.sprintf "from_gui: exception %s for message %s\n"
+	(Printexc2.to_string e) (GuiProto.string_of_from_gui t)))
 
 let gui_events () = 
   {
