@@ -84,7 +84,7 @@ let second_timer timer =
       update_link_stats ()
     with e ->
         lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
-	  (try
+  (try
      CommonUploads.refill_upload_slots ()
    with e ->
         lprintf_nl (_b "Exception %s") (Printexc2.to_string e));
@@ -102,7 +102,8 @@ let start_interfaces () =
   | Some dir -> allowed_ips =:= !!allowed_ips
   );
 
-  if !!http_port <> 0 then begin try
+  if !!http_port <> 0 then begin 
+      try
         ignore (DriverControlers.create_http_handler ());
       with e ->
           lprintf_nl (_b "Exception %s while starting HTTP interface")
@@ -218,7 +219,6 @@ let load_config () =
   (try
       Options.load downloads_ini;
       Options.load users_ini;
-      ignore (DriverInteractive.verify_user_admin ());
       DriverInteractive.hdd_check ()
     with e ->
         lprintf_nl "Exception %s during options load" (Printexc2.to_string e);
@@ -341,15 +341,18 @@ let _ =
       CommonGlobals.exit_properly 71
     end;
 
-  (
-    let hostname = "www.mldonkey.net" in
-    try
-      ignore(Ip.from_name hostname);
-      DriverInteractive.dns_works := true
-    with e ->
-        lprintf (_b "\nDNS resolution does not work! Looking up %s failed with %s.")
-           hostname (Printexc2.to_string e);
-	lprintf "
+  ( let resolve_name hostname =
+      try
+	ignore (Ip.from_name hostname);
+	true
+      with _ -> false
+    in
+    let hostnames =
+      ["www.mldonkey.org"; "mldonkey.sf.net"; "www.mldonkey.net"; "www.google.com"]
+    in
+    DriverInteractive.dns_works := List.exists resolve_name hostnames;
+
+    if not !DriverInteractive.dns_works then lprintf "
 The core therefore is unable to get eDonkey serverlists and loading
 .torrent files via dllink from websites is also impossible.
 If you are using MLDonkey in a chroot environment you should
@@ -382,6 +385,9 @@ or getting a binary compiled with glibc %s.\n\n")
 	end
       end
   );
+  if not !Charset.conversion_enabled then
+    lprintf_nl (_b "Self-test failed, charset conversion disabled.");
+
   load_config ();
   
   add_infinite_option_timer download_sample_rate CommonFile.sample_timer;
@@ -399,7 +405,7 @@ or getting a binary compiled with glibc %s.\n\n")
     save_results =:= old_save_results;
   end;
 
-  lprintf_nl (_b "Check http://www.mldonkey.net/ for updates");
+  lprintf_nl (_b "Check http://www.mldonkey.org for updates");
   networks_iter (fun r -> network_load_complex_options r);
   lprintf_nl (_b "enabling networks: ");
   networks_iter (fun r ->
@@ -413,7 +419,7 @@ or getting a binary compiled with glibc %s.\n\n")
   lprintf_nl (_b "---- enabling interfaces ----");
   List.iter (fun (p,s) -> if p <> 0 then lprintf_nl "using port %d (%s)" p s)
     (network_ports (network_find_by_name "Global Shares"));
-  lprintf (_b "%sdisabled networks: ") (log_time ());
+  lprintf (_b "%s[dMain] disabled networks: ") (log_time ());
   let found = ref false in
     networks_iter_all (fun r ->
         if not (network_is_enabled r) then
@@ -440,6 +446,25 @@ or getting a binary compiled with glibc %s.\n\n")
   add_infinite_timer 0.1 CommonUploads.upload_download_timer;
   add_infinite_timer !!buffer_writes_delay (fun _ -> Unix32.flush ());
 
+  history_timeflag := (Unix.time()); 
+  update_download_history (); 
+  update_upload_history ();
+  history_h_timeflag := (Unix.time()); 
+  update_h_download_history (); 
+  update_h_upload_history ();
+  history_size_for_h_graph := history_size * !!html_mods_vd_gfx_h_intervall / 60;
+  history_h_step := 60 * !!html_mods_vd_gfx_h_intervall;
+	
+  add_infinite_timer (float_of_int history_step) (fun timer -> 
+    history_timeflag := (Unix.time()); 
+    update_download_history (); 
+    update_upload_history ());
+
+  add_infinite_timer (float_of_int !history_h_step) (fun timer -> 
+    history_h_timeflag := (Unix.time()); 
+    update_h_download_history (); 
+    update_h_upload_history ());
+		
   if Autoconf.system = "mingw" then
     add_infinite_timer 1. (fun timer ->
         MlUnix.set_console_title (DriverInteractive.console_topic ()));
