@@ -47,6 +47,7 @@ let string_color_of_state state =
   | Connected_initiating -> M.fT_tx_initiating, Some !!O.color_not_connected
   | Connected 0 -> M.fT_tx_queued, Some !!O.color_connected
   | Connected n -> Printf.sprintf Gui_messages.fT_tx_ranked n, Some !!O.color_connected
+  | ServerFull -> "", None
   | NotConnected (_,n) -> 
       if n = -1 then
         "", None
@@ -72,7 +73,7 @@ let shorten maxlen s =
   if len > maxlen then
     (String.sub s 0 (maxlen-3)) ^ "..."
   else if s = "" then
-      "http://www.mldonkey.net/"
+      "http://www.mldonkey.org/"
   else s
 
 let state_pix state =
@@ -82,6 +83,7 @@ let state_pix state =
       | NewHost -> O.gdk_pix M.o_xpm_connect_n
       | Connected_initiating -> O.gdk_pix M.o_xpm_connect_m
       | Connected n -> O.gdk_pix M.o_xpm_connect_y
+      | ServerFull -> O.gdk_pix M.o_xpm_connect_n
       | NotConnected (_,n) -> O.gdk_pix M.o_xpm_connect_n
       | RemovedHost -> O.gdk_pix M.o_xpm_removedhost
       | BlackListedHost -> O.gdk_pix M.o_xpm_blacklistedhost
@@ -430,6 +432,7 @@ class box_friends box_files friend_tab =
         client_chat_port = 0;
         client_connect_time = c.gclient_connect_time;
         client_software = c.gclient_software;
+        client_os = None;
         client_release = c.gclient_release;
         client_emulemod = c.gclient_emulemod;
         client_downloaded = c.gclient_downloaded;
@@ -637,6 +640,7 @@ class box_list friend_tab =
         client_chat_port = 0;
         client_connect_time = c.gclient_connect_time;
         client_software = c.gclient_software;
+        client_os = None;
         client_release = c.gclient_release;
         client_emulemod = c.gclient_emulemod;
         client_downloaded = c.gclient_downloaded;
