@@ -31,6 +31,7 @@ open CommonComplexOptions
 open CommonServer
 open CommonResult
 open CommonFile
+open CommonShared
 open BasicSocket
 open CommonGlobals
 open Options
@@ -107,6 +108,45 @@ let (ft_ops : ft CommonFile.file_ops) =
 let (client_ops : client CommonClient.client_ops) =
   CommonClient.new_client_ops network
 
+let must_share_file file codedname has_old_impl =
+  match file.file_shared with
+  | Some _ -> ()
+  | None ->
+      begin
+        let impl = {
+          impl_shared_update = 1;
+          impl_shared_fullname = file_disk_name file;
+          impl_shared_codedname = codedname;
+          impl_shared_size = file_size file;
+          impl_shared_id = Md4.null;
+          impl_shared_num = 0;
+          impl_shared_uploaded = Int64.zero;
+          impl_shared_ops = shared_ops;
+          impl_shared_val = file;
+          impl_shared_requests = 0;
+          impl_shared_magic = None;
+          impl_shared_servers = [];
+        } in
+        file.file_shared <- Some impl;
+        incr CommonGlobals.nshared_files;
+        CommonShared.shared_calculate_total_bytes ();
+        match has_old_impl with
+          None -> update_shared_num impl
+        | Some old_impl -> replace_shared old_impl impl
+      end
+
+let must_share_file file = must_share_file file (file_best_name (as_file file)) None
+
+let unshare_file file =
+  match file.file_shared with
+    None -> ()
+  | Some s ->
+      begin
+        file.file_shared <- None;
+        decr CommonGlobals.nshared_files;
+        CommonShared.shared_calculate_total_bytes ()
+      end
+
 module DO = CommonOptions
 
 let current_files = ref ([] : BTTypes.file list)
@@ -126,10 +166,6 @@ let log_prefix = "[BT]"
 let lprintf_nl fmt =
   lprintf_nl2 log_prefix fmt
 
-let lprintf_file_nl  file fmt  =
-  lprintf_nl2 (log_prefix^" [file_num "^(string_of_int (file_num file))^"]") fmt
-
-    
 let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
@@ -217,13 +253,14 @@ let rec set_trackers file file_trackers =
             tracker_torrent_last_dl_req = 0;
             tracker_id = "";
             tracker_key = "";
-	    tracker_enabled = true
+	    tracker_status = Enabled
           } in
-	  t.tracker_enabled <- can_handle_tracker t;
+	  if not (can_handle_tracker t) then
+	    t.tracker_status <- Disabled_mld (intern "Tracker type not supported");
 	  file.file_trackers <-  t :: file.file_trackers;
 	set_trackers file q
 
-let new_file file_id t torrent_diskname file_temp file_state =
+let new_file file_id t torrent_diskname file_temp file_state user =
   try
     Hashtbl.find files_by_uid file_id
   with Not_found ->
@@ -233,7 +270,7 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_file = file_impl;
           file_piece_size = t.torrent_piece_size;
           file_id = file_id;
-          file_name = t.torrent_name;
+          file_name = Charset.safe_convert t.torrent_encoding t.torrent_name;
           file_comment = t.torrent_comment;
           file_created_by = t.torrent_created_by;
           file_creation_date = t.torrent_creation_date;
@@ -252,6 +289,8 @@ let new_file file_id t torrent_diskname file_temp file_state =
           file_shared = None;
         } and file_impl =  {
           dummy_file_impl with
+          impl_file_owner = user;
+          impl_file_group = user.user_default_group;
           impl_file_fd = Some file_fd;
           impl_file_size = t.torrent_length;
           impl_file_downloaded = Int64.zero;
@@ -296,18 +335,18 @@ let new_file file_id t torrent_diskname file_temp file_state =
       current_files := file :: !current_files;
       Hashtbl.add files_by_uid file_id file;
       file_add file_impl file_state;
-(*      lprintf "ADD FILE TO DOWNLOAD LIST\n"; *)
+      must_share_file file;
       file
 
-let new_download file_id t torrent_diskname =
+let new_download file_id t torrent_diskname user =
   let file_temp = Filename.concat !!DO.temp_directory
       (Printf.sprintf "BT-%s" (Sha1.to_string file_id)) in
-  new_file file_id t torrent_diskname file_temp FileDownloading
+  new_file file_id t torrent_diskname file_temp FileDownloading user
 
 let ft_by_num = Hashtbl.create 13
 let ft_counter = ref 0
 
-let new_ft file_name =
+let new_ft file_name user =
   incr ft_counter;
   let rec ft = {
       ft_file = file_impl;
@@ -316,6 +355,8 @@ let new_ft file_name =
       ft_retry = (fun _ -> ());
     } and file_impl =  {
       dummy_file_impl with
+      impl_file_owner = user;
+      impl_file_group = user.user_default_group;
       impl_file_fd = None;
       impl_file_size = zero;
       impl_file_downloaded = Int64.zero;
@@ -396,6 +437,7 @@ let decode_az_style s =
       | "AZ" -> Brand_azureus
       | "BB" -> Brand_bitbuddy
       | "BC" -> Brand_bitcomet
+      | "BR" -> Brand_bitrocket
       | "BS" -> Brand_btslave
       | "BX" -> Brand_bittorrentx
       | "CT" -> Brand_ctorrent
@@ -419,6 +461,11 @@ let decode_az_style s =
       | "TR" -> Brand_transmission
       | "HN" -> Brand_hydranode
       | "RT" -> Brand_retriever
+      | "PC" -> Brand_cachelogic
+      | "ES" -> Brand_electricsheep
+      | "qB" -> Brand_qbittorrent
+      | "QT" -> Brand_qt4
+      | "UL" -> Brand_uleecher
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
@@ -450,6 +497,7 @@ let decode_tornado_style s =
      | "A" -> Brand_abc
      | "U" -> Brand_upnp
      | "O" -> Brand_osprey
+     | "R" -> Brand_tribler
      | _ -> Brand_unknown
     in
     let bv = ref None in
@@ -742,7 +790,7 @@ let new_client file peer_id kind =
           client_downloaded = zero;
           client_upload_rate = Rate.new_rate ();
           client_downloaded_rate = Rate.new_rate ();
-          client_optimist_time=0;
+          client_connect_time = last_time ();
           client_blocks_sent = [];
           client_new_chunks = [];
           client_good = false;
@@ -753,6 +801,11 @@ let new_client file peer_id kind =
           client_incoming = false;
           client_registered_bitfield = false;
           client_last_optimist = 0;
+          client_dht = false;
+          client_cache_extension = false;
+          client_fast_extension = false;
+          client_utorrent_extension = false;
+          client_azureus_messaging_protocol = false;
         } and impl = {
           dummy_client_impl with
           impl_client_val = c;
@@ -789,6 +842,11 @@ let remove_tracker url file =
       lprintf_nl "New tracker list :%s" tracker.tracker_url
     ) file.file_trackers
 
+let tracker_is_enabled t =
+  match t.tracker_status with
+    Enabled -> true
+  | _ -> false
+
 let torrents_directory = "torrents"
 let new_torrents_directory = Filename.concat torrents_directory "incoming"
 let downloads_directory = Filename.concat torrents_directory "downloads"
