@@ -52,8 +52,34 @@ let _b x = _b "BTInteractive" x
 
 module VB = VerificationBitmap
 
-exception Already_exists
-exception Torrent_can_not_be_used
+let porttest_result = ref PorttestNotStarted
+
+let interpret_azureus_porttest s =
+  let failure_message fmt = 
+    Printf.sprintf ("Port test failure, " ^^ fmt) in
+  try
+    let value = decode s in 
+    match value with
+    | Dictionary alist ->
+	(try
+	   match List.assoc (String "result") alist with
+	   | Int 1L -> "Port test OK!"
+	   | Int 0L ->
+	       (try
+		  match List.assoc (String "reason") alist with
+		  | String reason -> failure_message "%s" reason
+		  | _ -> raise Not_found
+		with Not_found ->
+		  failure_message "%s" "no reason given")
+	   | Int status ->
+	       failure_message "unknown status code (%Ld)" status
+	   | _ -> raise Not_found
+	 with Not_found ->
+	   failure_message "%s" "no status given")
+    | _ ->
+	failure_message "unexpected value type %s" (Bencode.print value)
+  with _ ->
+    failure_message "%s" "broken bencoded value"
 
 let op_file_all_sources file =
   let list = ref [] in
@@ -101,7 +127,7 @@ let op_file_commit file new_name =
       set_file_state file FileShared;
 
       if Unix32.destroyed (file_fd file) then
-        if !verbose then lprintf_file_nl file "op_file_commit: FD is destroyed... repairing";
+        if !verbose then lprintf_file_nl (as_file file) "op_file_commit: FD is destroyed... repairing";
 
 (* During the commit operation, for security, the file_fd is destroyed. So
   we create it again to be able to share this file again. *)
@@ -110,7 +136,7 @@ let op_file_commit file new_name =
         (create_temp_file new_name (List.map (fun (file,size,_) -> (file,size)) file.file_files) (file_state file));
 
       if Unix32.destroyed (file_fd file) then
-        lprintf_file_nl file "op_file_commit: FD is destroyed... could not repair!";
+        lprintf_file_nl (as_file file) "op_file_commit: FD is destroyed... could not repair!";
 
       let new_torrent_diskname =
         Filename.concat seeded_directory
@@ -119,20 +145,20 @@ let op_file_commit file new_name =
       (try
           Unix2.rename file.file_torrent_diskname new_torrent_diskname;
         with _ ->
-          (lprintf_file_nl file "op_file_commit: failed to rename %s to %s"
+          (lprintf_file_nl (as_file file) "op_file_commit: failed to rename %s to %s"
               file.file_torrent_diskname new_torrent_diskname));
       file.file_torrent_diskname <- new_torrent_diskname;
 
     end 
 
-let op_file_print_html file buf =
-
-  html_mods_cntr_init ();
+let op_file_print file o =
 
+  let buf = o.conn_buf in
+  if use_html_mods o then begin
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Filename", "sr br", "Filename");
-    ("", "sr", file.file_name) ];
+    ("", "sr", (Charset.safe_convert file.file_encoding file.file_name)) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -143,24 +169,27 @@ let op_file_print_html file buf =
  ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-  html_mods_td buf [
-    ("Tracker(s)", "sr br", "Tracker(s)");
-    ("", "sr",
-      (let enabled_tracker_string = ref "" in
-       let disabled_tracker_string = ref "" in
-       List.iter (fun tracker ->
-	 if tracker.tracker_enabled then
-	   enabled_tracker_string := !enabled_tracker_string ^ (shorten tracker.tracker_url !!max_name_len) ^ " "
-	 else
-	   disabled_tracker_string := !disabled_tracker_string ^ (shorten tracker.tracker_url !!max_name_len) ^ " "
-      ) file.file_trackers;
-      (!enabled_tracker_string ^ (if !disabled_tracker_string <> "" then " - disabled: " ^ !disabled_tracker_string else "")))) ];
-
-  Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+  let tracker_header_printed = ref false in
+  List.iter (fun tracker ->
+    let tracker_text, tracker_error =
+      (match tracker.tracker_status with
+	  Disabled s | Disabled_mld s | Disabled_failure s -> 
+	    Printf.sprintf "disabled: %s" tracker.tracker_url, s
+	| _ -> tracker.tracker_url, "")
+    in
+    html_mods_td buf [
+      (if not !tracker_header_printed then
+        ("Tracker(s) (mouseover for errors)", "sr br", "Tracker(s)")
+       else
+	("", "sr br", ""));
+      (tracker_error, "sr", tracker_text)];
+    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+    tracker_header_printed := true;
+  ) file.file_trackers;
 
   html_mods_td buf [
     ("Torrent Filename", "sr br", "Torrent Fname");
-    ("", "sr", file.file_torrent_diskname) ];
+    ("", "sr", (Charset.safe_convert file.file_encoding file.file_torrent_diskname)) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
@@ -168,14 +197,14 @@ let op_file_print_html file buf =
     ("Comment", "sr br", "Comment");
     ("", "sr", match file.file_comment with
         "" -> "-"
-      | _ -> file.file_comment) ];
+      | _ -> (Charset.safe_convert file.file_encoding file.file_comment)) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
     ("Created by", "sr br", "Created by");
     ("", "sr", match file.file_created_by with
         "" -> "-"
-      | _ -> file.file_created_by) ];
+      | _ -> (Charset.safe_convert file.file_encoding file.file_created_by)) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -187,7 +216,7 @@ let op_file_print_html file buf =
     ("Modified by", "sr br", "Modified by");
     ("", "sr", match file.file_modified_by with
         "" -> "-"
-      | _ -> file.file_modified_by) ];
+      | _ -> (Charset.safe_convert file.file_encoding file.file_modified_by)) ];
 
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   html_mods_td buf [
@@ -200,7 +229,7 @@ let op_file_print_html file buf =
     match l with
       | [] -> ()
       | t :: q ->
-	  if not t.tracker_enabled then print_first_tracker q
+	  if not (tracker_is_enabled t) then print_first_tracker q
 	  else begin
 	    Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 	    html_mods_td buf [
@@ -262,24 +291,42 @@ let op_file_print_html file buf =
 	  end in
   print_first_tracker file.file_trackers;
 
+  (* This is bad.  Magic info should be automatically filled in when 
+     the corresponding chunks complete. (see CommonSwarming)
+
+     This code only fills in the magic info for subfiles when a user 
+     manually performs a "vd #".  (interfaces out of sync)
+  
+     Magic info for shared files with subfiles is missing as well?
+  *)
+  if !Autoconf.magic_works then begin
   let check_magic file =
     match Magic.M.magic_fileinfo file false with
       None -> None
-    | Some s -> Some (HashMagic.merge CommonGlobals.files_magic s)
+    | Some s -> Some (intern s)
   in
+    let fdn = file_disk_name file in
+    let new_file_files = ref [] in
+  
+    List.iter (fun (f, s, m) -> 
+      let subfile = Filename.concat fdn f in
+      new_file_files := (f,s, check_magic subfile) :: !new_file_files;
+    ) file.file_files;
+
+    file.file_files <- List.rev !new_file_files;
+    file_must_update file; (* Send update to guis *)
+
+  end;
+  (* -- End bad -- *)
+
   let cntr = ref 0 in
-  List.iter (fun (filename, size, _) ->
+  List.iter (fun (filename, size, magic) ->
     Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
     let fs = Printf.sprintf "File %d" !cntr in
     let magic_string =
-      if !Autoconf.magic_works then
-	begin
-          let subfile = Filename.concat (file_disk_name file) filename in
-          match check_magic subfile with
+      match magic with 
             None -> ""
-          | Some magic -> Printf.sprintf " / %s" magic
-	end
-      else ""
+      | Some m -> Printf.sprintf " / %s" m;
     in
     html_mods_td buf [
       (fs, "sr br", fs);
@@ -287,8 +334,68 @@ let op_file_print_html file buf =
     ];
     incr cntr;
   ) file.file_files
+  end else begin
 
-let op_file_print_sources_html file buf =
+  Printf.bprintf buf "Trackers:\n";
+  List.iter (fun tracker ->
+    match tracker.tracker_status with
+      Disabled s | Disabled_mld s | Disabled_failure s -> 
+	Printf.bprintf buf "%s, disabled: %s\n" tracker.tracker_url s
+      | _ -> Printf.bprintf buf "%s\n" tracker.tracker_url
+  ) file.file_trackers;
+  let s = Charset.safe_convert file.file_encoding file.file_torrent_diskname in
+  if s <> "" then Printf.bprintf buf "Torrent diskname: %s\n" s;
+  let s = Charset.safe_convert file.file_encoding file.file_comment in
+  if s <> "" then Printf.bprintf buf "Comment: %s\n" s;
+  let s = Charset.safe_convert file.file_encoding file.file_created_by in
+  if s <> "" then Printf.bprintf buf "Created by %s\n" s;
+  let s = Date.to_string (Int64.to_float file.file_creation_date) in
+  if s <> "" then Printf.bprintf buf "Creation date: %s\n" s;
+  let s = Charset.safe_convert file.file_encoding file.file_modified_by in
+  if s <> "" then Printf.bprintf buf "Modified by %s\n" s;
+  if file.file_encoding <> "" then Printf.bprintf buf "Encoding: %s\n" file.file_encoding;
+  if file.file_files <> [] then Printf.bprintf buf "Subfiles: %d\n" (List.length file.file_files);
+  let cntr = ref 0 in
+  List.iter (fun (filename, size, magic) ->
+    incr cntr;
+    let magic_string =
+      match magic with 
+	None -> ""
+      | Some m -> Printf.sprintf " / %s" m;
+    in
+    Printf.bprintf buf "File %d: %s (%Ld bytes)%s\n" !cntr filename size magic_string
+  ) file.file_files
+  end
+
+let op_file_print_sources file o =
+  let buf = o.conn_buf in
+
+(* redefine functions for telnet output *)
+  let html_mods_td buf l =
+    if use_html_mods o then
+      html_mods_td buf l
+    else
+      (* List *)
+      List.iter (fun (t,c,d)  ->
+	  (* Title Class Value *)
+	  Printf.bprintf buf "%s "
+	  d;
+      ) l
+  in
+  let html_mods_table_header buf n c l =
+    if use_html_mods o then
+      html_mods_table_header buf n c l
+    else
+      if List.length l > 0 then begin
+	Printf.bprintf buf "\n";
+	List.iter (fun (w,x,y,z)  ->
+	  (* Sort Class Title Value *)
+	  Printf.bprintf buf "%s "
+	  z;
+	) l;
+	Printf.bprintf buf "\n"
+    end
+  in
 
   if Hashtbl.length file.file_clients > 0 then begin
 
@@ -312,10 +419,15 @@ let op_file_print_sources_html file buf =
         ( "0", "srh ar", "Incoming [T]rue, [F]alse", "I" );
         ( "0", "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
 
-        ( "0", "srh ar", "Optimist Time", "O" );
+        ( "0", "srh ar", "Connect Time", "T" );
         ( "0", "srh ar", "Last optimist", "L.Opt" );
         ( "0", "srh br ar", "Num try", "N" );
 
+        ( "0", "srh", "DHT [T]rue, [F]alse", "D" );
+        ( "0", "srh", "Cache extensions [T]rue, [F]alse", "C" );
+        ( "0", "srh", "Fast extensions [T]rue, [F]alse", "F" );
+        ( "0", "srh", "uTorrent extensions [T]rue, [F]alse", "U" );
+        ( "0", "srh br", "Azureus messaging protocol [T]rue, [F]alse", "A" );
 (*
         ( "0", "srh", "Bitmap (absent|partial|present|verified)", (colored_chunks
         (Array.init (String.length info.G.file_chunks)
@@ -334,8 +446,10 @@ let op_file_print_sources_html file buf =
       html_mods_table_header buf "sourcesTable" "sources al" header_list;
 
       Hashtbl.iter (fun _ c ->
+	  if use_html_mods o then
           Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr());
 
+          let btos b = if b then "T" else "F" in
           let cc,cn = Geoip.get_country (fst c.client_host) in
           let td_list = [
             ("", "sr br ar", Printf.sprintf "%d" (client_num c));
@@ -346,25 +460,31 @@ let op_file_print_sources_html file buf =
             ] @ (if !Geoip.active then [( cn, "sr br", cc)] else []) @ [
             ("", "sr ar", (size_of_int64 c.client_uploaded));
             ("", "sr ar br", (size_of_int64 c.client_downloaded));
-            ("", "sr", (if c.client_interested then "T" else "F"));
-            ("", "sr", (if c.client_choked then "T" else "F"));
+            ("", "sr", (btos c.client_interested));
+            ("", "sr", (btos c.client_choked));
             ("", "sr br ar", (Int64.to_string c.client_allowed_to_write));
 (* This is way too slow for 1000's of chunks on a page with 100's of sources
     ("", "sr", (CommonFile.colored_chunks (Array.init (String.length c.client_bitmap)
        (fun i -> (if c.client_bitmap.[i] = '1' then 2 else 0)) )) );
 *)
-            ("", "sr", (if c.client_interesting then "T" else "F"));
-            ("", "sr", (if c.client_alrd_sent_interested then "T" else "F"));
-            ("", "br sr", (if c.client_alrd_sent_notinterested then "T" else "F"));
+            ("", "sr", (btos c.client_interesting));
+            ("", "sr", (btos c.client_alrd_sent_interested));
+            ("", "br sr", (btos c.client_alrd_sent_notinterested));
 
-            ("", "sr", (if c.client_good then "T" else "F"));
-            ("", "sr", (if c.client_incoming then "T" else "F"));
-            ("", "br sr", (if c.client_registered_bitfield then "T" else "F"));
+            ("", "sr", (btos c.client_good));
+            ("", "sr", (btos c.client_incoming));
+            ("", "br sr", (btos c.client_registered_bitfield));
 
-            ("", "sr", Printf.sprintf "%d" c.client_optimist_time);
+            ("", "sr", Printf.sprintf "%d" c.client_connect_time);
             ("", "ar sr", string_of_date c.client_last_optimist);
             ("", "br sr", Printf.sprintf "%d" c.client_num_try);
 
+            ("", "sr", (btos c.client_dht)); 
+            ("", "sr", (btos c.client_cache_extension)); 
+            ("", "sr", (btos c.client_fast_extension)); 
+            ("", "sr", (btos c.client_utorrent_extension)); 
+            ("", "br sr", (btos c.client_azureus_messaging_protocol)); 
+
             ("", "sr ar", (let fc = ref 0 in
                 (match c.client_bitmap with
                     None -> ()
@@ -374,19 +494,21 @@ let op_file_print_sources_html file buf =
           ] in
 
           html_mods_td buf td_list;
-          Printf.bprintf buf "\\</tr\\>";
+          if use_html_mods o then Printf.bprintf buf "\\</tr\\>"
+	  else Printf.bprintf buf "\n";
 
       ) file.file_clients;
 
-      Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>";
+      if use_html_mods o then Printf.bprintf buf "\\</table\\>\\</div\\>\\<br\\>"
+      else Printf.bprintf buf "\n";
 
     end
 
 let op_file_check file =
-  lprintf_file_nl file "Checking chunks of %s" file.file_name;
+  lprintf_file_nl (as_file file) "Checking chunks of %s" file.file_name;
   match file.file_swarmer with
     None ->
-      lprintf_file_nl file "verify_chunks: no swarmer to verify chunks"
+      lprintf_file_nl (as_file file) "verify_chunks: no swarmer to verify chunks"
   | Some swarmer ->
       CommonSwarming.verify_all_chunks_immediately swarmer
 
@@ -434,6 +556,7 @@ let op_file_info file =
     P.file_sub_files = file.file_files;
     P.file_active_sources = List.length (op_file_active_sources file);
     P.file_all_sources = (Hashtbl.length file.file_clients);
+    P.file_comment = file.file_comment;
   }
 
 let op_ft_info ft =
@@ -443,7 +566,7 @@ let op_ft_info ft =
   {
     P.file_fields = P.Fields_file_info.all;
 
-    P.file_comment = "";
+    P.file_comment = file_comment (as_ft ft);
     P.file_name = ft.ft_filename;
     P.file_num = ft_num ft;
     P.file_network = network.network_num;
@@ -465,11 +588,16 @@ let op_ft_info ft =
     P.file_priority = 0;
     P.file_uids = [];
     P.file_sub_files = [];
+    P.file_magic = None;
+    P.file_comments = [];
+    P.file_user = "";
+    P.file_group = "";
+    P.file_release = file_release (as_ft ft);
   }
 
 
 
-let load_torrent_string s =
+let load_torrent_string s user =
   let file_id, torrent = BTTorrent.decode_torrent s in
 
   (* Save the torrent, because we later want to put
@@ -488,20 +616,19 @@ let load_torrent_string s =
   if Sys.file_exists torrent_diskname then
     begin
       if !verbose then lprintf_nl "load_torrent_string: %s already exists, ignoring" torrent_diskname;
-      raise Already_exists
+      raise Torrent_already_exists
     end;
   File.from_string torrent_diskname s;
 
   if !verbose then
     lprintf_nl "Starting torrent download with diskname: %s"
         torrent_diskname;
-  let file = new_download file_id torrent torrent_diskname in
+  let file = new_download file_id torrent torrent_diskname user in
   BTClients.get_sources_from_tracker file;
-  BTShare.must_share_file file;
   CommonInteractive.start_download (file_find (file_num file));
   file
 
-let load_torrent_file filename =
+let load_torrent_file filename user =
   if !verbose then
     lprintf_nl "load_torrent_file %s" filename;
   let s = File.to_string filename in
@@ -510,27 +637,27 @@ let load_torrent_file filename =
   if Sys.file_exists filename
       && (Filename.dirname filename) = downloads_directory then
     Sys.remove filename;
-  ignore (load_torrent_string s)
+  ignore (load_torrent_string s user)
 
 let parse_tracker_reply file t filename =
 (*This is the function which will be called by the http client
 for parsing the response*)
 (* Interested only in interval*)
-  if !verbose_msg_servers then lprintf_file_nl file "Filename %s" filename;
+  if !verbose_msg_servers then lprintf_file_nl (as_file file) "Filename %s" filename;
     let tracker_reply =
       try
         File.to_string filename
-      with e -> lprintf_file_nl file "Empty reply from tracker"; ""
+      with e -> lprintf_file_nl (as_file file) "Empty reply from tracker"; ""
     in
     let v =
        match tracker_reply with
        | "" ->
         if !verbose_connect then
-          lprintf_file_nl file "Empty reply from tracker";
+          lprintf_file_nl (as_file file) "Empty reply from tracker";
         Bencode.decode ""
        | _ -> Bencode.decode tracker_reply
     in
-  if !verbose_msg_servers then lprintf_file_nl file "Received: %s" (Bencode.print v);
+  if !verbose_msg_servers then lprintf_file_nl (as_file file) "Received: %s" (Bencode.print v);
   t.tracker_interval <- 600;
   match v with
     Dictionary list ->
@@ -538,9 +665,9 @@ for parsing the response*)
           match (key, value) with
             String "interval", Int n ->
               t.tracker_interval <- Int64.to_int n;
-              if !verbose_msg_servers then lprintf_file_nl file ".. interval %d .." t.tracker_interval
+              if !verbose_msg_servers then lprintf_file_nl (as_file file) ".. interval %d .." t.tracker_interval
           | String "failure reason", String failure ->
-                lprintf_file_nl file "Failure from Tracker in file: %s Reason: %s"  file.file_name failure
+                lprintf_file_nl (as_file file) "Failure from Tracker in file: %s Reason: %s"  file.file_name failure
           (*TODO: merge with f from get_sources_from_tracker and parse the rest of the answer, too.
             also connect to the sources we receive or instruct tracker to send none, perhaps based
             on an config option. firewalled people could activate the option and then seed torrents, too.*)
@@ -559,7 +686,7 @@ let try_share_file torrent_diskname =
         match list with
           [] -> raise Not_found
         | sh :: tail ->
-            let s = sharing_strategies sh.shdir_strategy in
+            let s = sharing_strategy sh.shdir_strategy in
             if match torrent.torrent_files with
                 [] -> not s.sharing_directories
               | _ ->  s.sharing_directories then
@@ -576,9 +703,8 @@ let try_share_file torrent_diskname =
     in
 
     let file = new_file file_id torrent torrent_diskname
-        filename FileShared in
-    BTShare.must_share_file file;
-    if !verbose_share then lprintf_file_nl file "Sharing file %s" filename;
+        filename FileShared CommonUserDb.admin_user in
+    if !verbose_share then lprintf_file_nl (as_file file) "Sharing file %s" filename;
     BTClients.connect_trackers file "started"
       (parse_tracker_reply file)
   with
@@ -636,13 +762,17 @@ let scan_new_torrents_directory () =
     let file_basename = Filename.basename file in
     if not (Unix2.is_directory file) then
     try
-      load_torrent_file file;
+      let user = fst (Unix32.owner file) in
+      load_torrent_file file (try CommonUserDb.user2_user_find user with Not_found -> CommonUserDb.admin_user);
       (try Sys.remove file with _ -> ())
     with 
       Torrent_can_not_be_used ->
 	Unix2.rename file (Filename.concat old_directory file_basename);
 	lprintf_nl "Torrent %s does not have valid tracker URLs, moved to torrents/old ..." file_basename
-    | e -> lprintf_nl "Error %s in scan_new_torrents_directory for %s" (Printexc2.to_string e) file_basename
+    | e ->
+	Unix2.rename file (Filename.concat old_directory file_basename);
+	lprintf_nl "Error %s in scan_new_torrents_directory for %s, moved to torrents/old ..."
+	  (Printexc2.to_string e) file_basename
   ) filenames
 
 let retry_all_ft () =
@@ -651,12 +781,12 @@ let retry_all_ft () =
           lprintf_nl "ft_retry: exception %s" (Printexc2.to_string e)
   ) ft_by_num
 
-let load_torrent_from_web r ft =
+let load_torrent_from_web r user ft =
   let module H = Http_client in
   H.wget r (fun filename ->
       if ft_state ft = FileDownloading then begin
-          load_torrent_file filename;
-          file_cancel (as_ft ft)
+          load_torrent_file filename user;
+          file_cancel (as_ft ft) CommonUserDb.admin_user
         end)
 
 let valid_torrent_extension url =
@@ -669,7 +799,7 @@ let get_regexp_string text r =
   let b = Str.group_end 1 in
   String.sub text a (b - a)
 
-let op_network_parse_url url =
+let op_network_parse_url url user =
   let location_regexp = "Location: \\(.*\\)" in
   try
     let real_url = get_regexp_string url (Str.regexp location_regexp) in
@@ -700,13 +830,13 @@ let op_network_parse_url url =
                   ) "" cookies
                 ) ]
             with Not_found -> []);
-            H.req_max_retry = 10;
+	  H.req_max_retry = 10;
           } in
 
         let file_diskname = Filename.basename u.Url.short_file in
-        let ft = new_ft file_diskname in
-        ft.ft_retry <- load_torrent_from_web r ;
-        load_torrent_from_web r ft;
+        let ft = new_ft file_diskname user in
+        ft.ft_retry <- (load_torrent_from_web r user);
+        load_torrent_from_web r user ft;
         "started download", true
       )
     else
@@ -717,10 +847,10 @@ let op_network_parse_url url =
         try
           if !verbose then lprintf_nl "Not_found and trying to load %s" url;
           try
-            load_torrent_file url;
+            load_torrent_file url user;
             "", true
           with
-	    Already_exists -> "A torrent with this name is already in the download queue", false
+	    Torrent_already_exists -> "A torrent with this name is already in the download queue", false
 	  | Torrent_can_not_be_used -> "This torrent does not have valid tracker URLs", false
         with e ->
           lprintf_nl "Exception %s while 2nd loading" (Printexc2.to_string e);
@@ -753,6 +883,7 @@ let op_client_info c =
     P.client_downloaded = c.client_downloaded;
     P.client_uploaded = c.client_uploaded;
     P.client_upload = Some (c.client_file.file_name);
+    P.client_connect_time = c.client_connect_time;
 
   }
 
@@ -909,12 +1040,15 @@ let commands =
     ), _s ":\t\t\t\tprint all .torrent files on this server";
 
     "seeded_torrents", "Network/Bittorrent", Arg_none (fun o ->
+      if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
       List.iter (fun file ->
           if file_state file = FileShared then
               Printf.bprintf o.conn_buf "%s [%s]\n" file.file_name (Int64.to_string file.file_uploaded)
       ) !current_files;
       _s "done"
-
+      end else
+      begin print_command_result o o.conn_buf "You are not allowed to use seeded_torrents";
+      "" end
     ), _s ":\t\t\tprint all seeded .torrent files on this server";
 
     "reshare_torrents", "Network/Bittorrent", Arg_none (fun o ->
@@ -950,13 +1084,13 @@ let commands =
       let buf = o.conn_buf in
       if Sys.file_exists url then
         begin
-          load_torrent_file url;
+	  load_torrent_file url o.conn_user.ui_user;
           Printf.bprintf buf "loaded file %s\n" url
 	end
       else
         begin
           let url = "Location: " ^ url ^ "\nContent-Type: application/x-bittorrent" in
-          let result = fst (op_network_parse_url url) in
+	  let result = fst (op_network_parse_url url o.conn_user.ui_user) in
           Printf.bprintf buf "%s\n" result
 	end;
       _s ""
@@ -986,7 +1120,7 @@ let commands =
           Hashtbl.iter (fun _ file ->
               if file_num file = num then begin
                   if !verbose then
-                    lprintf_file_nl file "adding trackers for file %i" num;
+                    lprintf_file_nl (as_file file) "adding trackers for file %i" num;
                   set_trackers file !urls;
                   raise Exit
                 end
@@ -1031,12 +1165,18 @@ let commands =
 open LittleEndian
 open GuiDecoding
 
-let op_gui_message s =
+let op_gui_message s user =
   match get_int16 s 0 with
     0 ->
       let text = String.sub s 2 (String.length s - 2) in
       if !verbose then lprintf_nl "received torrent from gui...";
-      ignore (load_torrent_string text)
+      (try
+        ignore (load_torrent_string text user)
+      with e -> (match e with
+	  Torrent_can_not_be_used -> lprintf_nl "Loading torrent from GUI: this torrent can not be used"
+	| Torrent_already_exists -> lprintf_nl "Loading torrent from GUI: this torrent is already in download queue"
+	| _ -> ());
+	raise e)
   | 1 -> (* 34+ *)
       let n = get_int s 2 in
       let a, pos = get_string s 6 in
@@ -1057,12 +1197,17 @@ let _ =
   file_ops.op_file_active_sources <- op_file_active_sources;
   file_ops.op_file_debug <- op_file_debug;
   file_ops.op_file_commit <- op_file_commit;
-  file_ops.op_file_print_html <- op_file_print_html;
-  file_ops.op_file_print_sources_html <- op_file_print_sources_html;
+  file_ops.op_file_print <- op_file_print;
+  file_ops.op_file_print_sources <- op_file_print_sources;
   file_ops.op_file_check <- op_file_check;
   file_ops.op_file_cancel <- op_file_cancel;
   file_ops.op_file_info <- op_file_info;
   file_ops.op_file_save_as <- (fun file name -> ());
+  file_ops.op_file_shared <- (fun file ->
+      match file.file_shared with
+        None -> None
+      | Some sh -> Some (as_shared sh)
+  );
 
   network.op_network_gui_message <- op_gui_message;
   network.op_network_connected <- op_network_connected;
@@ -1072,7 +1217,7 @@ let _ =
   network.op_network_forget_search <- (fun s -> ());
   network.op_network_connect_servers <- (fun s -> ());
   network.op_network_search <- (fun ss buf -> ());
-  network.op_network_download <- (fun r -> dummy_file);
+  network.op_network_download <- (fun r user -> dummy_file);
   network.op_network_recover_temp <- (fun s -> ());
   let clean_exit_started = ref false in
   network.op_network_clean_exit <- (fun s ->
@@ -1090,7 +1235,25 @@ let _ =
     !!client_port, "client_port TCP";
     !!BTTracker.tracker_port, "tracker_port TCP";
     ]);
-
+  network.op_network_porttest_result <- (fun _ -> !porttest_result);
+  network.op_network_porttest_start <- (fun _ -> 
+      let module H = Http_client in
+      azureus_porttest_random := (Random.int 100000);
+      porttest_result := PorttestInProgress (last_time ());
+      let r = {
+          H.basic_request with
+          H.req_url =
+	    Url.of_string (Printf.sprintf
+	      "http://azureus.aelitis.com/natcheck.php?port=%d&check=azureus_rand_%d"
+		!!client_port !azureus_porttest_random);
+          H.req_proxy = !CommonOptions.http_proxy;
+          H.req_user_agent = get_user_agent ();
+        } in
+      H.wget r (fun file ->
+	let result = interpret_azureus_porttest (File.to_string file) in
+	porttest_result := PorttestResult (last_time (), result)
+      )
+  );
   client_ops.op_client_info <- op_client_info;
   client_ops.op_client_connect <- op_client_connect;
   client_ops.op_client_disconnect <- op_client_disconnect;
@@ -1101,8 +1264,8 @@ let _ =
   CommonNetwork.register_commands commands;
 
   shared_ops.op_shared_unshare <- (fun file ->
-      (if !verbose_share then lprintf_file_nl file "unshare file");
-      BTShare.unshare_file file);
+      (if !verbose_share then lprintf_file_nl (as_file file) "unshare file");
+      BTGlobals.unshare_file file);
   shared_ops.op_shared_info <- (fun file ->
    let module T = GuiTypes in
      match file.file_shared with
