@@ -182,9 +182,6 @@ let value_to_server assocs =
     ) in
   let l = DonkeyGlobals.new_server ip port in
   
-  (try
-      l.server_description <- get_value "server_desc" value_to_string 
-    with _ -> ());
   (try
       l.server_name <- get_value "server_name" value_to_string
     with _ -> ());
@@ -193,7 +190,7 @@ let value_to_server assocs =
     with _ -> ());
   (try
       connection_set_last_conn l.server_connection_control
-        (normalize_time (mini (get_value "server_age" value_to_int) 
+        (normalize_time (min (get_value "server_age" value_to_int) 
           (BasicSocket.last_time ())));
     with _ -> ());
   as_server l.server_server
@@ -207,11 +204,6 @@ let server_to_value c =
   ]
   in
   
-  let fields = 
-    if c.server_description <> "" then 
-      ("server_desc", string_to_value c.server_description) :: fields
-    else fields in
-  
   let fields = 
     if c.server_name <> "" then 
       ("server_name", string_to_value c.server_name) :: fields
@@ -273,7 +265,7 @@ let value_to_file file_size file_state assocs =
   in
 
   let file = DonkeyGlobals.new_file file_diskname file_state
-    (Md4.of_string file_md4) file_size "" true in
+    (Md4.of_string file_md4) file_size "" true CommonUserDb.admin_user in
 
   (try
       set_file_best_name (as_file file)
@@ -563,8 +555,10 @@ let result_of_file md4 tags =
       match tag with
         { tag_name = Field_Filename; tag_value = String s } ->
           r.result_names <- s :: r.result_names
-      | { tag_name = Field_Size; tag_value = Uint64 v } ->
+      | { tag_name = Field_Size; tag_value = (Uint64 v| Fint64 v) } ->
           r.result_size <- v;
+      | { tag_name = Field_Size; tag_value = (Uint16 v| Uint8 v) } ->
+          r.result_size <- Int64.of_int v;
       | { tag_name = Field_Format; tag_value = String s } ->
           r.result_tags <- tag :: r.result_tags;
           r.result_format <- s
