@@ -69,7 +69,6 @@ let quarter_timer timer =
   clean_join_queue_tables ()
 
 let fivemin_timer timer =
-  DonkeyShare.send_new_shared ();
   clients_root := []
 
 let second_timer timer =
@@ -79,13 +78,13 @@ let second_timer timer =
         if !verbose_sources > 2 then 
           lprintf_nl "Exception %s while checking sources" 
             (Printexc2.to_string e)
-  );
-  DonkeyServers.udp_walker_timer ()
+  )
 
 let five_second_timer timer =
   DonkeyServers.check_server_connections ();
   DonkeyServers.walker_timer ();
   DonkeyOneFile.check_files_downloaded ();
+  DonkeyServers.udp_walker_timer ();
   DonkeyShare.check_shared_files ()
 
 let min_timer timer =
@@ -95,10 +94,7 @@ let min_timer timer =
  (try
       DonkeyServers.query_locations_timer ();
     with _ -> ());
-  (List.iter (fun file -> DonkeyShare.must_share_file file) !new_shared_files;
-  new_shared_files := [])
-(*  DonkeyIndexer.add_to_local_index_timer () *)
-
+  DonkeyShare.send_new_shared ()
 
 let local_login () =
   if !!login = "" then !!global_login else !!login
@@ -118,10 +114,6 @@ let disable enabler () =
         | Some sock -> 
             listen_sock := None;
             TcpServerSocket.close sock Closed_by_user);
-      (match !reversed_sock with None -> ()
-        | Some sock -> 
-            reversed_sock := None;
-            TcpServerSocket.close sock Closed_by_user);
       (match !udp_sock with None -> ()
         | Some sock -> 
             udp_sock := None;
@@ -136,11 +128,13 @@ let reset_tags () =
   let module D = DonkeyProtoClient in
   let m = D.mldonkey_emule_proto in
 
-  let secident = if !!enable_sui then 3 else 0 in
+  let secident = if sec_ident_enabled () then 3 else 0 in
     m.emule_secident <- secident;
     m.emule_features <- secident;
 
   let emule_miscoptions1 = D.emule_miscoptions1 m in
+  let emule_miscoptions2 = D.emule_miscoptions2 m in
+  let emule_compatoptions = D.emule_compatoptions m in
   client_to_client_tags :=
   [
     string_tag (Field_UNKNOWN "name") (local_login ());
@@ -149,20 +143,40 @@ let reset_tags () =
     int_tag (Field_UNKNOWN "emule_udpports") (!!donkey_port+4);
     int_tag (Field_UNKNOWN "emule_version") m.emule_version;
     int64_tag (Field_UNKNOWN "emule_miscoptions1") emule_miscoptions1;
+    int64_tag (Field_UNKNOWN "emule_miscoptions2") emule_miscoptions2;
+    int_tag (Field_UNKNOWN "emule_compatoptions") emule_compatoptions;
   ];
 
-  let extended = ref 0x04 in (* support of auxport *)
-    extended := !extended lor 0x01; (* support of compression *)
+(* server capabilities *)
+  let extended = ref 0x0 in
+  extended := !extended lor 0x01; (* support of compression *)
+(*extended := !extended lor 0x02;    IP in login, deprecated *)
+  extended := !extended lor 0x04; (* support of auxport *)
+  extended := !extended lor 0x08; (* newtags *)
+(*extended := !extended lor 0x10; (* unicode *) *)
+(*extended := !extended lor 0x100; (* files > 4GB *) *)
+(*extended := !extended lor 0x200; (* support crypt *) *)
+(*extended := !extended lor 0x400; (* request crypt *) *)
+(*extended := !extended lor 0x800; (* require crypt *) *)
 
   client_to_server_tags :=
   [
     string_tag (Field_UNKNOWN "name") (local_login ());
     int_tag (Field_UNKNOWN "version") protocol_version;
-    int_tag (Field_UNKNOWN "port") !!donkey_port;
     int_tag (Field_UNKNOWN "extended") !extended;
     int_tag (Field_UNKNOWN "emule_version") m.emule_version;
   ];
 
+  client_to_server_reply_tags :=
+  [
+    string_tag (Field_UNKNOWN "name") (local_login ());
+    int_tag (Field_UNKNOWN "version") protocol_version;
+    int_tag (Field_UNKNOWN "emule_udpports") (!!donkey_port+4);
+    int64_tag (Field_UNKNOWN "emule_miscoptions1") emule_miscoptions1;
+    int64_tag (Field_UNKNOWN "emule_miscoptions2") emule_miscoptions2;
+    int_tag (Field_UNKNOWN "emule_version") m.emule_version;
+  ];
+
   emule_info.DonkeyProtoClient.EmuleClientInfo.tags <- [
     int_tag (Field_UNKNOWN "compression") m.emule_compression;
     int_tag (Field_UNKNOWN "udpver") m.emule_udpver;
@@ -302,9 +316,7 @@ be useful when users want to share files that they had already previously
       
       Options.option_hook global_login reset_tags;
       Options.option_hook login reset_tags;
-      Options.option_hook enable_sui ( fun _ ->
-	if not (Autoconf.donkey_sui_works ()) && !!enable_sui then enable_sui =:= false;
-	reset_tags ());
+      Options.option_hook enable_sui reset_tags;
 				
 (**** START TIMERS ****)
       add_session_option_timer enabler check_client_connections_delay 
@@ -380,10 +392,10 @@ let _ =
             [] -> "" | opfile :: _ -> options_file_name opfile);
         network_netname = network.network_name;
         network_netflags = network.network_flags;
-        network_enabled = network.op_network_is_enabled ();
+        network_enabled = network_is_enabled network;
         network_uploaded = !donkey_upload_counter;
         network_downloaded = !donkey_download_counter;
-        network_connected = List.length (connected_servers ());
+        network_connected_servers = List.length (connected_servers ());
       });
   network.op_network_ports <- (fun _ ->
     [
@@ -392,6 +404,8 @@ let _ =
     !overnet_port_info, "overnet_port TCP+UDP";
     !kademlia_port_info, "kademlia_port UDP";
     ]);
+  network.op_network_porttest_result <-
+    (fun _ -> !DonkeyInteractive.porttest_result);
   CommonInteractive.register_gui_options_panel "eDonkey" 
     gui_donkey_options_panel;
   CommonInteractive.register_gui_options_panel "Overnet" 
