@@ -51,6 +51,7 @@ let mldonkey_emule_proto =
     emule_secident = 3; (* Emule uses v1 if advertising both, v2 if only advertising 2 *)
     emule_noviewshared = 0;
     emule_supportpreview = 0;
+    emule_osinfosupport = 1;
     emule_compression = 1; (* 1 *)
     emule_sourceexchange = 2; (* 2 : +client_md4 3 : +IdHybrid (emule Kademlia?)*)
     emule_multipacket = 0; (* 1 *)
@@ -73,6 +74,15 @@ let emule_miscoptions1 m =
   in
   Int64.of_int o
 
+let emule_miscoptions2 m =
+(*
+  let o =
+    (m.emule_largefiles lsl 4)
+  in
+  Int64.of_int o
+*)
+  Int64.zero
+
 let update_emule_proto_from_miscoptions1 m o =
   let o = Int64.to_int o in
   m.emule_udpver <- (o lsr 24) land 0xf;
@@ -85,6 +95,18 @@ let update_emule_proto_from_miscoptions1 m o =
   m.emule_multipacket <- (o lsr 1) land 0x1;
   m.emule_supportpreview <- (o lsr 0) land 0x1
 
+let update_emule_proto_from_miscoptions2 m o = ()
+(*
+  let o = Int64.to_int o in
+  m.emule_largefiles <- (o lsr 4) land 0x1
+*)
+
+let emule_compatoptions m =
+  (m.emule_osinfosupport lsl 0)
+
+let update_emule_proto_from_compatoptions m o =
+  m.emule_osinfosupport <- (o lsr 0) land 0x1
+
 let extendedrequest e =
   min e.emule_extendedrequest mldonkey_emule_proto.emule_extendedrequest
 
@@ -133,6 +155,7 @@ module Connect  = struct
         "\060", Field_UNKNOWN "downloadtime";
         "\061", Field_UNKNOWN "incompleteparts";
         "\085", Field_UNKNOWN "mod_version";
+        "\239", Field_UNKNOWN "emule_compatoptions";
         "\249", Field_UNKNOWN "emule_udpports";
         "\250", Field_UNKNOWN "emule_miscoptions1";
         "\251", Field_UNKNOWN "emule_version";
@@ -799,7 +822,7 @@ module EmuleClientInfo = struct
         "\135", "mod_lsd";
         "\136", "mod_lsd_version";
         "\144", "mod_lovelace_version";
-        "\148", "mod_oxy";
+        "\148", "os_info"; (* reused by aMule to transfer client OS type *)
         "\153", "mod_plus";
         "\160", "mod_wombat";
         "\161", "dev_wombat";
@@ -1044,6 +1067,43 @@ module EmuleRequestSourcesReply = struct
       done
 
   end
+module EmuleFileDesc = struct
+
+    type t = {
+        rating : int;
+        comment : string;
+      }
+
+    let parse len s =
+      let rating = get_uint8 s 1 in
+      let (comment, _) = get_string32 s 2 in
+      {
+        rating = rating;
+        comment = comment;
+      }
+
+    let print t =
+      lprintf_nl "EmuleFileDesc [%d][%s]" t.rating t.comment
+
+    let write buf t =
+      buf_int8 buf t.rating;
+      buf_string buf t.comment
+  end
+
+
+module EmulePortTestReq = struct
+
+    type t = string
+
+    let print s =
+      lprintf_nl "Emule porttest request %s" (String.escaped s)
+
+    let parse s = s
+
+    let write buf =
+      buf_int8 buf 0x12
+
+  end
 
 type t =
 | ConnectReq of Connect.t
@@ -1081,13 +1141,14 @@ type t =
 | EmuleQueueRankingReq of EmuleQueueRanking.t
 | EmuleRequestSourcesReq of EmuleRequestSources.t
 | EmuleRequestSourcesReplyReq of EmuleRequestSourcesReply.t
-| EmuleFileDescReq of int * string
+| EmuleFileDescReq of EmuleFileDesc.t 
 | EmulePublicKeyReq of EmulePublicKeyReq.t
 | EmuleSignatureReq of EmuleSignatureReq.t
 | EmuleSecIdentStateReq  of EmuleSecIdentStateReq.t
 | EmuleMultiPacketReq of Md4.t * t list
 | EmuleMultiPacketAnswerReq of Md4.t * t list
 | EmuleCompressedPart of Md4.t * int64 * int64 * string
+| EmulePortTestReq of EmulePortTestReq.t
 
 let rec print t =
   begin
@@ -1133,8 +1194,8 @@ let rec print t =
     | EmuleRequestSourcesReplyReq t ->
         EmuleRequestSourcesReply.print t
 
-    | EmuleFileDescReq (rating, comment) ->
-        lprintf "EMULE FILE DESC %s" comment
+    | EmuleFileDescReq t ->
+        EmuleFileDesc.print t
 
     | EmuleMultiPacketReq (md4, list) ->
         lprintf_nl "EmuleMultiPacket for %s:" (Md4.to_string md4);
@@ -1155,11 +1216,11 @@ let rec print t =
         EmuleSignatureReq.print t
     | EmulePublicKeyReq t ->
         EmulePublicKeyReq.print t
-
     | EmuleCompressedPart (md4, statpos, newsize, bloc) ->
         lprintf_nl "EmuleCompressedPart for %s %Ld %Ld len %d"
           (Md4.to_string md4) statpos newsize (String.length bloc)
-
+    | EmulePortTestReq t ->
+        EmulePortTestReq.print t
     | UnknownReq (opcode, s) ->
         let len = String.length s in
         lprintf_nl "UnknownReq: magic (%d), opcode (%d) len (%d)" opcode 
@@ -1195,10 +1256,7 @@ let rec parse_emule_packet emule opcode len s =
 
     | 0x60 (* 96 *) -> EmuleQueueRankingReq (EmuleQueueRanking.parse len s)
 
-    | 0x61 (* 97 *) ->
-        let rating = get_uint8 s 1 in
-        let (comment,_) = get_string32 s 2 in
-        EmuleFileDescReq (rating, comment)
+    | 0x61 (* 97 *) -> EmuleFileDescReq (EmuleFileDesc.parse len s)
 
     | 0x81 (* 129 *) -> EmuleRequestSourcesReq (EmuleRequestSources.parse len s)
     | 0x82 (* 130 *) ->
@@ -1294,6 +1352,9 @@ let rec parse_emule_packet emule opcode len s =
         in
         EmuleMultiPacketAnswerReq (md4, iter s 17 len)
 
+    | 0xfe (* 254 *) ->
+        EmulePortTestReq s
+
     | code ->
         if !CommonOptions.verbose_unknown_messages then
           lprintf_nl "EDK: unknown eMule message %d" code;
@@ -1514,11 +1575,9 @@ let write emule buf t =
     | EmuleRequestSourcesReplyReq t ->
         buf_int8 buf 0x82;
         EmuleRequestSourcesReply.write emule buf t
-    | EmuleFileDescReq (rating, comment) ->
+    | EmuleFileDescReq t ->
         buf_int8 buf 0x61;
-        buf_int8 buf rating;
-        buf_string buf comment
-
+        EmuleFileDesc.write buf t
     | EmuleCompressedPart (md4, statpos, newsize, bloc) ->
         buf_int8 buf 0x40;
         buf_md4 buf md4;
@@ -1578,6 +1637,10 @@ let write emule buf t =
         buf_int8 buf 0x85;
         EmulePublicKeyReq.write buf t
 
+    | EmulePortTestReq t ->
+        buf_int8 buf 0xfe;
+        EmulePortTestReq.write buf;
+
     | UnknownReq (opcode, s) ->
         Buffer.add_string buf s
 
