@@ -39,6 +39,7 @@ type emule_proto = {
     mutable emule_secident : int;
     mutable emule_noviewshared : int;
     mutable emule_supportpreview : int;
+    mutable emule_osinfosupport : int;
 
     mutable emule_compression : int;
     mutable emule_sourceexchange : int;
@@ -430,14 +431,27 @@ module DonkeySources = CommonSources.Make(struct
         | Invalid_address _ -> failwith "Invalid address"
     end)
 
-type server = (*[]*){
+type file = {
+    file_file : file CommonFile.file_impl;
+    file_md4 : Md4.t;
+    mutable file_swarmer : CommonSwarming.t option;
+    mutable file_nchunks : int;
+    mutable file_nchunk_hashes : int;
+    mutable file_diskname : string;
+    mutable file_computed_md4s : Md4.t array;
+    mutable file_format : format;
+    mutable file_shared : file CommonShared.shared_impl option;
+    mutable file_sources : DonkeySources.file_sources_manager;
+    mutable file_comments : (Ip.t * string * int * string) list;
+  }
+
+and server = (*[]*){
     mutable server_server : server CommonServer.server_impl;
     mutable server_ip : Ip.t;
     mutable server_cid : Ip.t option;
     mutable server_port : int;
     mutable server_realport : int option; (* in case we connect through auxport; this is the true one *)
     mutable server_sock : tcp_connection;
-    mutable server_nqueries : int;
     mutable server_search_queries : CommonTypes.search Fifo.t;
     mutable server_users_queries : bool Fifo.t;
     mutable server_connection_control : connection_control;
@@ -452,10 +466,14 @@ type server = (*[]*){
     mutable server_next_udp : int;
     mutable server_master : bool;
     mutable server_preferred : bool;
-    mutable server_last_message : int; (* used only by mldonkey server *)
-
-    mutable server_last_ping : float;
+    mutable server_last_ping : float; (* time last PingServerUdpReq was sent *)
+    mutable server_next_ping : float; (* time next PingServerUdpReq will be sent *)
+    mutable server_descping_counter : int;
     mutable server_ping : int;
+    mutable server_udp_ping_challenge : int64 option;
+    mutable server_udp_desc_challenge : int64 option;
+(* reset after answer received, increased after UDP ping sent, server deleted after 10 attempts *)
+    mutable server_failed_count : int; 
 
     mutable server_id_requests : file option Fifo.t;
 
@@ -463,6 +481,18 @@ type server = (*[]*){
     mutable server_waiting_queries : file list;
     mutable server_sent_all_queries : bool;
     mutable server_has_zlib : bool;
+    mutable server_has_newtags : bool;
+    mutable server_has_unicode : bool;
+    mutable server_has_related_search : bool;
+    mutable server_has_tag_integer : bool;
+    mutable server_has_largefiles : bool;
+    mutable server_obfuscation_tcp : int option;
+    mutable server_obfuscation_udp : int option;
+    mutable server_dynip : string;
+    mutable server_auxportslist : string;
+    mutable server_has_get_sources : bool;
+    mutable server_has_get_files : bool;
+    mutable server_has_get_sources2 : bool;
 
     mutable server_flags : int;
     mutable server_version : string;
@@ -470,6 +500,7 @@ type server = (*[]*){
     mutable server_soft_limit : int64 option;
     mutable server_hard_limit : int64 option;
     mutable server_max_users : int64 option;
+    mutable server_sent_shared : file list
   }
 
 
@@ -519,7 +550,6 @@ and client = {
       CommonSwarming.uploader
       ) list;
     mutable client_all_files : result list option;
-    mutable client_last_asked_file : CommonTypes.file;
     mutable client_tags: CommonTypes.tag list;
     mutable client_name : string;
     mutable client_rating : int ;
@@ -531,6 +561,8 @@ and client = {
     mutable client_uploaded : Int64.t;
     mutable client_brand : brand;
     mutable client_brand_mod : brand_mod;
+    mutable client_osinfo_sent : bool;
+    mutable client_osinfo : string option;
     mutable client_banned : bool;
     mutable client_score : int;
     mutable client_next_queue : int;
@@ -549,6 +581,7 @@ and client = {
     mutable client_req_challenge : Int64.t;
     mutable client_public_key : string option;
     mutable client_sui_verified : bool option;
+    mutable client_last_file_req_md4 : Md4.t option;
   }
 
 and compressed_parts = {
@@ -579,19 +612,6 @@ and client_kind =
   int * (* last connection attempt *)
   int   (* booked client num *)
 
-and file = {
-    file_file : file CommonFile.file_impl;
-    file_md4 : Md4.t;
-    mutable file_swarmer : CommonSwarming.t option;
-    mutable file_nchunks : int;
-    mutable file_nchunk_hashes : int;
-    mutable file_diskname : string;
-    mutable file_computed_md4s : Md4.t array;
-    mutable file_format : format;
-    mutable file_shared : file CommonShared.shared_impl option;
-    mutable file_sources : DonkeySources.file_sources_manager;
-  }
-
 and file_to_share = {
     shared_name : string;
     shared_size : int64;
@@ -674,6 +694,7 @@ let dummy_emule_proto = {
     emule_secident = 0;
     emule_noviewshared = 0;
     emule_supportpreview = 0;
+    emule_osinfosupport = 0;
 
     emule_compression = 0; (* 1 *)
     emule_sourceexchange = 0; (* 3 *)
