@@ -204,15 +204,9 @@ let string_of_reason c =
 let minf (x: float) (y: float) =
   if x > y then y else x
 
-let mini (x: int) (y: int) =
-  if x > y then y else x
-
 let maxf (x: float) (y: float) =
   if x < y then y else x
 
-let maxi (x: int) (y: int) =
-  if x < y then y else x
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         Some functions                                *)
